﻿/**
 * class CTimer
 * @param {int} interval
 * @param {function} actionListener
 */
var CTimer = function(interval,actionListener){
	this._timer = null;
	this._interval = interval;
	this._actionListener = actionListener;
	this._isAlive = false;
	this._startTime = 0;
}
CTimer.prototype = {
	isAlive:function(){
		return this._isAlive;
	},
	interrupt:function(){
		if(this._timer)
			clearTimeout(this._timer);
		this._timer = null;
		this._isAlive = false;
		this._startTime = 0;
	},
	setActionListener:function(l){
		if(typeof l == "function")
			this._actionListener = l;
	},
	start:function(){
		var _this = this;
		this._timer = setTimeout(function(){
			_this.interrupt();
			_this._actionListener.call(_this);
		},this._interval);
		this._startTime = (new Date()).getTime();
		this._isAlive = true;
	},
	getElapsed:function(){
		return (new Date()).getTime() - this._startTime;
	},
	setInterval:function(interval){
		if(typeof interval == "number")
			this._interval = interval;
	},
	getInterval:function(){
		return this._interval;
	}
}
/**
 * class Lang
 */
var Lang = {
	ARRAY:0,
	BOOLEAN:1,
	DATE:2,
	ERROR:3,
	FUNCTION:4,
	NUMBER:5,
	OBJECT:6,
	REGEXP:7,
	STRING:8,
	UNDEFINED:9,
	NAN:10,
	NULL:11,
	/**
	 * argument type checking
	 * @param {Object} arg
	 * @param {int|Object} type
	 * @param {boolean} raiseException
	 */
	ensureType:function(arg,type,raiseException){
		var approved = false;
		switch(type){
			case this.ARRAY:
				approved = (arg instanceof Array);break;
			case this.BOOLEAN:
				approved = (typeof arg == "boolean");break;
			case this.DATE:
				approved = (arg instanceof Date);break;
			case this.FUNCTION:
				approved = (typeof arg == "function");break;
			case this.NUMBER:
				approved = (typeof arg == "number");break;
			case this.OBJECT:
				approved = (typeof arg == "object");break;
			case this.REGEXP:
				approved = (arg instanceof RegExp);break;
			case this.STRING:
				approved = (typeof arg == "string");break;
			case this.UNDEFINED:
				approved = (typeof arg == "undefined");break;
			case this.NAN:
				approved = (arg == Number.NaN);break;
			case this.NULL:
				approved = (arg === null);break;
			default:
				if(typeof type != "number" && type instanceof Object)
					approved = (arg instanceof type);
		}
		if(!approved && raiseException === true)
			throw new Error("Inappropriate Argument:"+arg);
		else
			return approved;
	},
	/**
	 * assert
	 * @param {Object} arg assertion declaration
	 * @param {Object} err the message
	 */
	assert:function(arg,err){
		var assertResult =	this.ensureType(arg,this.BOOLEAN);
		if(assertResult && !arg)
			console.error(typeof err == "undefined" ? "assertion failed in function calling at: "+arguments.callee.caller : err);
	}
}
String.prototype.contains = function(target){
	return Lang.ensureType(target,Lang.STRING) && this.indexOf(target) != -1;
}
String.prototype.empty = function(){
	return this.trim().length == 0;
}
String.prototype.startsWith = function(target){
	return Lang.ensureType(target,Lang.STRING) ? (this.indexOf(target)==0) : false;
}
String.prototype.endsWith = function(target){
	return Lang.ensureType(target,Lang.STRING) ? (this.lastIndexOf(target)==this.length-target.length) : false;
}
/**
 * Resource Provider
 */
const R = {
    exp:{
        IPv4:/\d{1,3}(?:(?:\.\d{1,3}){3})/gi,
        Number:/^\d+$/gi,
        Email:/^([^\s@]+)@([^\s@]+)$/gi,
        URL:/^(((?:[\w-_]+):\/\/)*)([\w-_]+)((?:\.[\w-_])+).*$/gi,
        UriScheme:/^[\w-]+:/gi
    }
}
/**
 * class CContextProvider
 */
var CContextProvider = function(){
	this._canvas = null;
	this._toast = null;
	this._persistenceResolver = null;
	this._i18n = null;
}
CContextProvider.instance = null;
CContextProvider.getDefaultProvider = function(){
	return CContextProvider.instance ? CContextProvider.instance : (CContextProvider.instance = new CContextProvider());
}
CContextProvider.SERVICE_CANVAS = 0;
CContextProvider.SERVICE_TOAST = 1;
CContextProvider.SERVICE_PERSISTENCE_RESOLVER = 2;
CContextProvider.SERVICE_I18N_RESOURCE = 3;
CContextProvider.prototype = {
	getService:function(s){
		switch(s){
			case CContextProvider.SERVICE_CANVAS:
				return this._canvas ? this._canvas : (this._canvas = new CCanvas());
			case CContextProvider.SERVICE_TOAST:
			    if(!this._toast){
			        this._toast = new CToast();
			        var f = document.createElement("div");
			        document.body.appendChild(f);
			        f.innerHTML = this._toast.toHTMLString();
			        this._toast.initUI();
			    }
			    return this._toast;
			case CContextProvider.SERVICE_PERSISTENCE_RESOLVER:
			    if(!this._persistenceResolver)
			        this._persistenceResolver =  new PersistenceResolver();
			    return this._persistenceResolver;
			case CContextProvider.SERVICE_I18N_RESOURCE:
			   return  this._i18n ? this._i18n : (this._i18n = new LocalStrings());
			default:
				return null;
		}
	}
}
/**
 * class ClassAttributeList
 * @param {Node} node
 */
var ClassAttributeList = function(node){
	this._node = node;
}
ClassAttributeList.prototype = {
	add:function(cls){
		if(this._validityCheck(cls) && !this.has(cls)){
			var oldCls = this._node.className.trim();
			this._node.className = oldCls.empty() ? cls : oldCls+" "+cls;
		}
		return this;
	},
	has:function(cls){
		if(!this._validityCheck(cls))
			return false;
		return this._node.className.contains(cls);
	},
	remove:function(cls){
		if(this._validityCheck(cls) && this.has(cls)){
			var list = this._node.className.split(/\s+/g);
			list.splice(list.indexOf(cls),1);
			this._node.className = list.join(" ");
		}
		return this;
	},
	replace:function(cls,newCls){
		if(this._validityCheck(cls) && this._validityCheck(newCls) && this.has(cls))
			this._node.className = this._node.className.replace(cls,newCls);
		return this;
	},
	toggle:function(cls){
		if(this.has(cls))
			this.remove(cls);
		else
			this.add(cls);
	},
	_validityCheck:function(cls){
		if(!cls)
			return false;
		return true;
	}
}
Node.PATTERN_CAMEL = /(\w+)-(\w+)/gi;
Node.prototype.currentStyle = function(property){
	if(typeof property != "string" || property.empty())
		return null;
	Node.PATTERN_CAMEL.lastIndex = 0;
	if(Node.PATTERN_CAMEL.test(property))
		property = RegExp.$1+RegExp.$2.charAt(0).toUpperCase()+RegExp.$2.substring(1);
	var style = document.defaultView.getComputedStyle(this,null);
	return style ? style[property] : null;
}
Node.prototype.setStyle = function(property,value){
	if(typeof property != "string" || property.empty())
		return null;
	Node.PATTERN_CAMEL.lastIndex = 0;
	if(Node.PATTERN_CAMEL.test(property))
		property = RegExp.$1+RegExp.$2.charAt(0).toUpperCase()+RegExp.$2.substring(1);
	this.style[property] = value;
}
//Node.prototype.classList = function(){
//return new ClassAttributeList(this);
//}
if(!("classList" in Node.prototype)){
	Node.prototype.__defineGetter__("classList",function(){
		return new ClassAttributeList(this);
	});
}
/**
 * class CConst
 */
var CConst = {
	NORTH:1,
	SOUTH:2,
	WEST:3,
	EAST:4,
	NORTH_EAST:5,
	SOUTH_EAST:6,
	NORTH_WEST:7,
	SOUTH_WEST:8,
	CENTER:9,
	HORTIZONTAL:1,
	VERTICAL:2
}
/**
 * abstract class CComponent
 * it's the fundamental component from which all component
 * want to work together with the framework
 * should derive itself.
 */
var CComponent = function(){
	this._eventListenerMap = {};
	this._UIId = CComponent.generateUID();
	
	this._cssClassComponents = [];
	this._insets = null;
	this._annotation = null;
	this._enabled = true;
	//
	// attribute data
	//
	this._attributeElements = {};
}
CComponent._uniqueId = 0;
CComponent.generateUID = function(){
	CComponent._uniqueId++;
	return "chromeplus_ui_"+CComponent._uniqueId;
}
CComponent.prototype = {
	/**
	 * addPropertyEventListener
	 * @param {string} property
	 * @param {function} l
	 */
	addPropertyEventListener:function(property,l){
		Lang.ensureType(l,Lang.FUNCTION,true);
		Lang.ensureType(property,Lang.STRING,true);
		if(property.empty())
			throw new Error("Missing Property Event Type");
		var listeners = this._eventListenerMap[property];
		if(typeof listeners == "undefined")
			listeners = [];
		listeners.push(l);
		this._eventListenerMap[property] = listeners;
	},
	removePropertyEventListener:function(property,l){
		var listeners = this._eventListenerMap[property];
		if(typeof listeners != "undefined"){
			for(var i=0,len=listeners.length;i<len;i++){
				if(l === listeners[i]){
					listeners.splice(i,1);
					break;
				}
			}
		}
	},
	firePropertyEvent:function(property,arg){
		var listeners = this._eventListenerMap[property];
		//assume that it has a positive result naively
		var proceed = true;
		if(typeof listeners != "undefined"){
			for(var i=0,len=listeners.length;i<len;i++){
				proceed = proceed && listeners[i].call(this,this,property,arg);
			}
		}
		return proceed;
	},
	getUIId:function(){
		return this._UIId;
	},
	addCssClassComponent:function(css){
		if(this._cssClassComponents.indexOf(css) == -1)
			this._cssClassComponents.push(css);
	},
	setAttribute:function(key,value){
	    this._attributeElements[key] = value;
	},
	getAttribute:function(key){
	    return this._attributeElements[key];
	},
	/**
	 * abstract method
	 * @param {boolean} b
	 */
	setEnabled:function(b){
	},
	enabled:function(){
		return this._enabled;
	},
	/**
	 * depending on concrete implementation
	 */
	toHTMLString:function(){
		return "";
	},
	/**
	 * depending on concrete implementation
	 */
	initUI:function(){
	},
	setAnnotation:function(annotation){
		Lang.ensureType(annotation,CAnnotation,true);
		this._annotation = annotation;
	},
	getAnnotation:function(){
		return this._annotation;
	},
	setInsets:function(top,right,bottom,left){
		if(!Lang.ensureType(top,Lang.NUMBER) || 
			!Lang.ensureType(right,Lang.NUMBER) || 
				!Lang.ensureType(bottom,Lang.NUMBER) || 
					!Lang.ensureType(left,Lang.NUMBER))
			return;
		if(arguments.length == 4){
			this._insets = [top+"px",right+"px",bottom+"px",left+"px"];
		}
	}
}

/**
 * abstract class CContainer
 */
var CContainer = function(){
	CComponent.call(this);
	this._components = [];
	this._DOMContainerUI = null;
}
CContainer.prototype = {
	__proto__:CComponent.prototype,
	add:function(component){
		Lang.ensureType(component,CComponent,true);
		//append
		if(this._components.indexOf(component) == -1)
			this._components.push(component);
	},
	remove:function(component){
		for(var i=0,len=this._components.length;i<len;i++){
			if(component == this._components[i]){
				this._components.splice(i,1);
				break;
			}
		}
	},
	getComponents:function(){
		return this._components;
	},
	getComponentAt:function(i){
		return (i in this._components) ? this._components[i] : null;
	},
	getComponentCount:function(){
		return this._components.length;
	},
	getComponentByUIId:function(id){
		if(!Lang.ensureType(id,Lang.STRING) || id.empty())
			return null;
		for(var i=0,len=this._components.length;i<len;i++){
			if(this._components[i] == id)
				return this._components[i];
		}
		return null;
	},
	toHTMLString:function(){
		var html = [];
		for(var i=0,len=this._components.length;i<len;i++)
			html.push(this._components[i].toHTMLString());
		return html.join("");
	},
	setEnabled:function(b){
		if(Lang.ensureType(b,Lang.BOOLEAN)){
			this._enabled = b;
			//update children enabled status
			this.setChildrenEnabled(b);
		}
	},
	setChildrenEnabled:function(b){
		if(Lang.ensureType(b,Lang.BOOLEAN)){
			for(var i=0,len=this.getComponentCount();i<len;i++)
				this.getComponentAt(i).setEnabled(b);
		}
	},
	/**
	 * abstract depending on concrete implementation
	 */
	getDOMContainer:function(){
		return null;
	},
	/**
	 * abstract cause the container to lay out it's children again
	 * it should be invoke after some modification are made to <code>CContainer</code>'s
	 * children
	 */
	validate:function(){
	},
	initUI:function(){
		this._components.forEach(function(element,index){
			element.initUI();
		});
	}
}
/**
 * abstract class CWindow
 */
var CWindow = function(){
	CContainer.call(this);
	//propety
	this._size = [0,0];
	this._location = [0,0];
	this._z = 0;
	this._isVisible = false;
}
CWindow.EVENT_IDENTIFIER_COMPONENT = "C_WINDOW_COMPONENT";
CWindow.prototype = {
	__proto__:CContainer.prototype,
	addComponentListener:function(l){
		this.addPropertyEventListener(CWindow.EVENT_IDENTIFIER_COMPONENT,l);
	},
	removeComponentListener:function(l){
		this.removePropertyEventListener(CWindow.EVENT_IDENTIFIER_COMPONENT,l);
	},
	fireComponentEvent:function(){
		this.firePropertyEvent(CWindow.EVENT_IDENTIFIER_COMPONENT);
	},
	setSize:function(w,h){
	},
	getSize:function(){
		return this._size;
	},
	setLocation:function(left,top){
	},
	isVisible:function(){
		return this._isVisible;
	},
	setVisible:function(){
	},
	getLocation:function(){
		return this._location;
	},
	setAlign:function(c){
		switch(c){
			case CConst.CENTER:
				var viewportWidth = Math.max(document.body.clientWidth,document.documentElement.clientWidth);
				var viewportHeight = Math.max(document.body.clientHeight,document.documentElement.clientHeight);
				this.setLocation(
					(viewportWidth-this._size[0])/2,
					(viewportHeight-this._size[1])/2
				);
				break;
			default:
		}
	},
	setZIndex:function(z){
	},
	getZIndex:function(){
		return this._z;
	}
}
/**
 * classs CFloatWindow
 * @param {String} title
 */
var CFloatWindow = function(title){
	Lang.ensureType(title,Lang.STRING,true);
	CWindow.call(this);
	this._title = title;
	this._domUI = null;
}
CFloatWindow.prototype = {
	__proto__:CWindow.prototype,
	setSize:function(w,h){
		Lang.ensureType(w,Lang.NUMBER,true);
		Lang.ensureType(h,Lang.NUMBER,true);
		this._domUI.setStyle("width",w+"px");
		this._domUI.setStyle("height",h+"px");
		this._size = [w,h];
	},
	setVisible:function(b){
		Lang.ensureType(b,Lang.BOOLEAN,true);
		if(this._size[0] == 0)
			this._size = [this._domUI.offsetWidth,this._domUI.offsetHeight];
		if(b){
			this._domUI.classList.remove("chrome-ui-window-disappear");
		}else{
			this._domUI.classList.add("chrome-ui-window-disappear");
		}
	},
	setHeadingText:function(heading){
		Lang.ensureType(heading,Lang.STRING,true);
		document.querySelector("#"+this._UIId+" .chromeplus-ui-window-heading").textContent = heading;
	},
	setLocation:function(left,top){
		Lang.ensureType(left,Lang.NUMBER,true);
		Lang.ensureType(top,Lang.NUMBER,true);
		this._domUI.setStyle("left",left+"px");
		this._domUI.setStyle("top",top+"px");
		this._location = [left,top];
	},
	setZIndex:function(z){
		Lang.ensureType(z,Lang.NUMBER,true);
		this._domUI.setStyle("z-index",z);
		this._z = z;
	},
	toHTMLString:function(){
		var children = CContainer.prototype.toHTMLString.call(this);
		return "<div"+(this._insets?" style=\"margin:"+this._insets.join(" ")+"\"":"")+" id=\""+this._UIId+"\" class=\"chromeplus-ui chromeplus-ui-window chromeplus-ui-float-window chrome-ui-window-disappear"+(this._cssClassComponents.length?" "+this._cssClassComponents.join(" "):"")+"\"><h3 class=\"chromeplus-ui-window-heading\">"+this._title+"</h3><a class=\"chromeplus-ui-window-close\" href=\"javascript:void(0)\" title=\"Close\">✖</a><div class=\"chromeplus-ui-window-content pseudo-clearfix\">"+children+"</div></div>";
	},
	getDOMUI:function(){
		return this._domUI;
	},
	initUI:function(){
		CContainer.prototype.initUI.call(this);
		var _this = this;
		this._domUI = document.querySelector("#"+this._UIId);
		document.body.appendChild(this._domUI);
		this._domUI.querySelector(".chromeplus-ui-window-close").addEventListener("click",function(){
			_this.setVisible(false);
		},false);
		this._domUI.addEventListener("webkitTransitionEnd",function(){
			_this._isVisible = !this.classList.contains("chrome-ui-window-disappear");
			_this.fireComponentEvent();
		},false);
	}
}
/**
 * class CCanvas
 */
var CCanvas = function(){
	CFloatWindow.call(this,"");
	this.initUI();
}
CCanvas.prototype = {
	__proto__:CFloatWindow.prototype,
	toHTMLString:function(){
		return "";
	},
	setVisible:function(b){
		Lang.ensureType(b,Lang.BOOLEAN,true);
		if(this._size[0] == 0)
			this._size = [this._domUI.offsetWidth,this._domUI.offsetHeight];
		if(b){
			this._domUI.setStyle("display","block");
			this.setEnclosedWithWindow();
		}else{
			this._domUI.setStyle("display","none");
		}
		this._isVisible = b;
	},
	setEnclosedWithWindow:function(){
		var h = Math.max(document.documentElement.scrollHeight,document.documentElement.clientHeight);
		var w = Math.max(document.documentElement.scrollWidth,document.documentElement.clientWidth);
		this.setSize(w,h);
	},
	initUI:function(){
		var _this = this;
		this._domUI = document.createElement("div");
		this._domUI.id = this._UIId;
		this._domUI.className = "chromeplus-ui chromeplus-ui-canvas";
		document.body.appendChild(this._domUI);
		window.addEventListener("resize",function(){
			if(_this.isVisible())
				_this.setEnclosedWithWindow();
		},false);
	}
}
/**
 * abstract class CCheckableButton
 */
var CCheckableButton = function(label){
	CContainer.call(this);
	this._label = label;
	
	this._domControlName = null;
}
CCheckableButton.EVENT_IDENTIFIER_ACTION = "C_CHECKABLE_BUTTON_ACTION";
CCheckableButton.prototype = {
	__proto__:CContainer.prototype,
	setEnabled:function(b){
		if(Lang.ensureType(b,Lang.BOOLEAN)){
			CContainer.prototype.setEnabled.call(this,b);
			if(this._domUI)
				this._domUI.disabled = !b;
		}
	},
	setChecked:function(b){
		if(Lang.ensureType(b,Lang.BOOLEAN) && this._domUI)
			this._domUI.checked = b;
	},
	checked:function(){
		return this._domUI ? this._domUI.checked : false;
	},
	/**
	 * this method is supposed to get called by group object
	 * @param {string} name
	 */
	setDOMControlName:function(name){
		if(Lang.ensureType(name,Lang.STRING) && !name.empty())
			this._domControlName = name;
	},
	addActionListener:function(l){
		this.addPropertyEventListener(CCheckableButton.EVENT_IDENTIFIER_ACTION,l);
	},
	removeActionListener:function(l){
		this.removePropertyEventListener(CCheckableButton.EVENT_IDENTIFIER_ACTION,l);
	},
	fireActionEvent:function(){
		this.firePropertyEvent(CCheckableButton.EVENT_IDENTIFIER_ACTION);
	},
	initUI:function(){
		CContainer.prototype.initUI.call(this);
		var DOMUI = document.querySelector("#dom_"+this._UIId);
		if(DOMUI){
			this._domUI = DOMUI;
			var _this = this;
			this._domUI.addEventListener("click",function(){
				_this.fireActionEvent();
			},false);
		}
	},
	getDOMUI:function(){
		return this._domUI;
	}
}

/**
 * class CRadioButton
 * @param {String} label
 */
var CRadioButton = function(label){
	CCheckableButton.call(this,label);
	this._domUI = null;
}
CRadioButton.prototype = {
	__proto__:CCheckableButton.prototype,
	toHTMLString:function(){
		if(this._annotation)
			this.addCssClassComponent("chromeplus-ui-what-is-this");
		//super.toHTMLString
		var childrenHTML = CCheckableButton.prototype.toHTMLString.call(this);
		//assemble html string
		return "<div"+(this._insets?" style=\"margin:"+this._insets.join(" ")+"\"":"")+" id=\""+this._UIId+"\" class=\"chromeplus-ui chromeplus-ui-container chromeplus-ui-radiobutton"+(this._cssClassComponents.length?" "+this._cssClassComponents.join(" "):"")+"\"><label><input type=\"radio\" name=\""+(this._domControlName||this.getUIId())+"\" id=\"dom_"+this.getUIId()+"\">"+this._label+"</label>"+(this._annotation?this._annotation.toHTMLString():"")+childrenHTML+"</div>";
	}
}
/**
 * class CCheckbox
 * @param {String} label
 */
var CCheckbox = function(label){
	CCheckableButton.call(this,label);
	this._domUI = null;
}
CCheckbox.prototype = {
	__proto__:CCheckableButton.prototype,
	/**
	 * @override
	 */
	toHTMLString:function(){
		if(this._annotation)
			this.addCssClassComponent("chromeplus-ui-what-is-this");
		//super.toHTMLString
		var childrenHTML = CCheckableButton.prototype.toHTMLString.call(this);
		//assemble html string
		return "<div"+(this._insets?" style=\"margin:"+this._insets.join(" ")+"\"":"")+" id=\""+this._UIId+"\" class=\"chromeplus-ui chromeplus-ui-container chromeplus-ui-checkbox"+(this._cssClassComponents.length?" "+this._cssClassComponents.join(" "):"")+"\"><label><input type=\"checkbox\" name=\""+(this._domControlName||this.getUIId())+"\" id=\"dom_"+this.getUIId()+"\">"+this._label+"</label>"+(this._annotation?this._annotation.toHTMLString():"")+childrenHTML+"</div>";
	}
}

/**
 * class CSlider
 * @param {string} legend
 * @param {int} min
 * @param {int} max
 * @param {int} step
 * @param {int} value optional
 */
var CSlider = function(legend,min,max,step,value){
	CComponent.call(this);
	Lang.ensureType(legend,Lang.STRING,true/**raise exception if necessary**/);
	Lang.ensureType(min,Lang.NUMBER,true);
	Lang.ensureType(max,Lang.NUMBER,true);
	Lang.ensureType(step,Lang.NUMBER,true);
	
	this._legend = legend;
	this._min = min;
	this._max = max;
	this._step = step;
	this._value = Lang.ensureType(value,Lang.NUMBER) ? value : min;
	
	this._dynamicValue = this._value;
	
	this._domUI = null;
	this._domMonitorUI = null;
}
CSlider.EVENT_IDENTIFIER_CHANGE = "C_SLIDER_VALUE_CHANGE";
CSlider.prototype = {
	__proto__:CComponent.prototype,
	getMinium:function(min){
		return this._min;
	},
	getMaximum:function(max){
		return this._max;
	},
	getStep:function(step){
		return this._step;
	},
	getValue:function(){
		return this._domUI ? this._domUI.value : null;
	},
	setValue:function(value){
		if(typeof value == "number"){
			this._dynamicValue = value;
			this._domUI.value = value;
			this.valueChanged(value);
		}
	},
	/**
	 * protected valueChanged
	 * @param {int} v
	 */
	valueChanged:function(v){
		this._domMonitorUI.textContent = v;
		this._dynamicValue = v;
		this.fireChangeEvent();
	},
	addChangeListener:function(l){
		this.addPropertyEventListener(CSlider.EVENT_IDENTIFIER_CHANGE,l);
	},
	removeChangeListener:function(l){
		this.removePropertyEventListener(CSlider.EVENT_IDENTIFIER_CHANGE,l);
	},
	fireChangeEvent:function(){
		this.firePropertyEvent(CSlider.EVENT_IDENTIFIER_CHANGE);
	},
	setEnabled:function(b){
		if(Lang.ensureType(b,Lang.BOOLEAN)){
			this._enabled = b;
			if(this._domUI)
				this._domUI.disabled = !b;
		}
	},
	getDOMUI:function(){
		return this._domUI;
	},
	initUI:function(){
		var _this = this;
		this._domUI = document.querySelector("#"+this._UIId+" input[type='range']");
		this._domUI.onchange = function(){
			_this.valueChanged(parseInt(this.value));
		}
		this._domMonitorUI = this._domUI.nextSibling;
	},
	toHTMLString:function(){
		if(this._annotation)
			this.addCssClassComponent("chromeplus-ui-what-is-this");
		return "<div"+(this._insets?" style=\"margin:"+this._insets.join(" ")+"\"":"")+" id=\""+this._UIId+"\" class=\"chromeplus-ui chromeplus-ui-range"+(this._cssClassComponents.length?" "+this._cssClassComponents.join(" "):"")+"\"><h4 class=\"chromeplus-ui-range-title\">"+this._legend+"</h4><label><input type=\"range\" step=\""+this._step+"\" value=\""+this._value+"\" min=\""+this._min+"\" max=\""+this._max+"\" name=\"dom_"+this._UIId+"\" id=\"dom_"+this._UIId+"\" /><dfn class=\"chromeplus-ui-range-monitor\">"+this._value+"</dfn></label>"+(this._annotation?this._annotation.toHTMLString():"")+"</div>";
	}
}

/**
 * class CSlate
 * @param {string} legend
 */
var CSlate = function(legend){
	CContainer.call(this);
	this._legendText = legend;
	
	this._freeHTML = null;
}
CSlate.EVENT_IDENTIFIER_DOM_READY = "C_SLATE_DOM_READY";
CSlate.prototype = {
	__proto__:CContainer.prototype,
	setFreeHTML:function(html){
		Lang.ensureType(html,Lang.STRING,true);
		this._freeHTML = html;
	},
	addSeparator:function(){
		this.add(new CSeparator());
	},
	/**
	 * slate as a higher level container can receive raw events other than semantic events
	 * @param {function} l
	 */
	addDOMReadyListener:function(l){
		this.addPropertyEventListener(CSlate.EVENT_IDENTIFIER_DOM_READY,l);
	},
	removeDOMReadyListener:function(l){
		this.removePropertyEventListener(CSlate.EVENT_IDENTIFIER_DOM_READY,l);
	},
	fireDOMReadyEvent:function(){
		this.firePropertyEvent(CSlate.EVENT_IDENTIFIER_DOM_READY);
	},
	toHTMLString:function(){
		var childrenHTML = this._freeHTML || CContainer.prototype.toHTMLString.call(this);
		var html = ["<div"+(this._insets?" style=\"margin:"+this._insets.join(" ")+"\"":"")+" id=\""+this._UIId+"\" class=\"chromeplus-ui chromeplus-ui-slat chromeplus-ui-container"+(this._cssClassComponents.length?" "+this._cssClassComponents.join(" "):"")+"\">"];
		html.push("<h3 class=\"chromeplus-ui-slat-title\">"+(this._legendText?this._legendText:"")+"</h3>");
		html.push("<div class=\"chromeplus-ui-slat-content\">"+childrenHTML+"</div></div>");
		return html.join("");
	},
	/**
	 * @override
	 */
	getDOMContainer:function(){
		return this._DOMContainerUI ? this._DOMContainerUI : (this._DOMContainerUI = document.querySelector("#"+this._UIId+">.chromeplus-ui-slat-content"));
	},
	initUI:function(){
		var _this = this;
		//invoke super.initUI();
		CContainer.prototype.initUI.call(this);
		this.fireDOMReadyEvent();
	}
}
/**
 * abstract class CSequenceComponentGroup
 * @param int mode a layout constraint
 */
var CSequenceComponentGroup = function(mode,selectionMode,legend){
	CContainer.call(this);
	this._mode = mode;
	this._selectionMode = selectionMode;
	this._selectedIndex = -1;
	this._precedingSelectedIndex = -1;
	
	this._legendText = legend;
	
	this._inLibraryActionListener = null;
}
CSequenceComponentGroup.SELECTION_MODE_EXCLUSIVE = 0;
CSequenceComponentGroup.SELECTION_MODE_INCLUSIVE = 1;
CSequenceComponentGroup.EVENT_IDENTIFIER_SELECTION = "C_SEQUENCE_COMPONENT_GROUP_SELECTION";
/**
 * static newCheckBoxGroup
 * @param {int} mode
 * @param {int} selectionMode
 * @param {String} legend
 */
CSequenceComponentGroup.newCheckBoxGroup = function(mode,selectionMode,legend){
	return new CCheckBoxGroup(mode,selectionMode,legend);
}
/**
 * newRadioButtonGroup
 * @param {int} layout mode
 * @param {String} legend text
 */
CSequenceComponentGroup.newRadioButtonGroup = function(mode,legend){
	return new CRadioButtonGroup(mode,legend);
}
CSequenceComponentGroup.prototype = {
	__proto__:CContainer.prototype,
	/**
	 * @override
	 */
	add:function(component){
		if(component instanceof CCheckableButton){
			//assign in-library listener
			var _this = this;
			if(!this._inLibraryActionListener){
				this._inLibraryActionListener = function(com){
					for(var list=_this.getComponents(),len=list.length,i=0;i<len;i++){
						if(list[i].getUIId() == com.getUIId()){
							_this.itemSelected(i);
							break;
						}
					}
				}
			}
			component.addActionListener(this._inLibraryActionListener);
			CContainer.prototype.add.call(this,component);
		}
	},
	addSelectionListener:function(l){
		this.addPropertyEventListener(CSequenceComponentGroup.EVENT_IDENTIFIER_SELECTION,l);
	},
	removeSelectionListener:function(l){
		this.removePropertyEventListener(CSequenceComponentGroup.EVENT_IDENTIFIER_SELECTION,l);
	},
	fireSelectionEvent:function(){
		this.firePropertyEvent(CSequenceComponentGroup.EVENT_IDENTIFIER_SELECTION);
	},
	getSelectedIndex:function(){
		return this._selectedIndex;
	},
	getPrecedingSelectedIndex:function(){
		return this._precedingSelectedIndex;
	},
	/**
	 * protected itemSelected
	 * @param {int} index
	 * @param {boolean} is fire event
	 */
	itemSelected:function(index,suppressEvent){
	},
	setSelected:function(index,suppressEvent){
	},
	toHTMLString:function(){
		if(this._annotation)
			this.addCssClassComponent("chromeplus-ui-what-is-this");
		var childrenHTML = CContainer.prototype.toHTMLString.call(this);
		var html = ["<div"+(this._insets?" style=\"margin:"+this._insets.join(" ")+"\"":"")+" id=\""+this._UIId+"\" class=\"chromeplus-ui "+(this._mode==CConst.VERTICAL?"chromeplus-ui-layout-vertical":"chromeplus-ui-layout-horizontal")+" chromeplus-ui-container chromeplus-ui-sequence-group"+(this._cssClassComponents.length?" "+this._cssClassComponents.join(" "):"")+"\">"];
		if(this._legendText)
			html.push("<h3 class=\"chromeplus-ui-sequence-group-legend\">"+this._legendText+(this._annotation?this._annotation.toHTMLString():"")+"</h3>");
		html.push(childrenHTML+"</div>");
		return html.join("");
	},
	setLegendText:function(text){
		if(typeof text == "string" && !text.empty())
			this._legendText = text;
	},
	initUI:function(){
		//call super.initUI
		CContainer.prototype.initUI.call(this);
	}
}

/**
 * class CRadioButtonGroup
 * @param {int} mode
 * @param {string} legend
 */
var CRadioButtonGroup = function(mode,legend){
	CSequenceComponentGroup.call(this,mode,CSequenceComponentGroup.SELECTION_MODE_EXCLUSIVE,legend);
}
CRadioButtonGroup.prototype = {
	__proto__:CSequenceComponentGroup.prototype,
	add:function(component){
		if(component instanceof CRadioButton){
			//proceed something
			component.setDOMControlName("dom_control_"+this._UIId);
			CSequenceComponentGroup.prototype.add.call(this,component);
		}
	},
	/**
	 * protected itemSelected
	 * @param {int} index
	 */
	itemSelected:function(index,suppressEvent){
		if(index == this._selectedIndex)
			return;
		this._precedingSelectedIndex = this._selectedIndex;
		this._selectedIndex = index;
		if(suppressEvent !== true)
			this.fireSelectionEvent();
	},
	setSelected:function(index,suppressEvent){
		var components = this.getComponents();
		if(index in components){
			components[index].setChecked(true);
			this.itemSelected(index,suppressEvent);
		}
	},
	initUI:function(){
		CSequenceComponentGroup.prototype.initUI.call(this);
		this.setSelected(0);
	}
}

/**
 * class CCheckBoxGroup
 * @param {init} mode
 * @param {int} selectionMode
 * @param {string} legend
 */
var CCheckBoxGroup = function(mode,selectionMode,legend){
	CSequenceComponentGroup.call(this,mode,selectionMode,legend);
}
CCheckBoxGroup.prototype = {
	__proto__:CSequenceComponentGroup.prototype,
	/**
	 * @override
	 */
	add:function(component){
		if(component instanceof CCheckbox){
			//proceed something
			CSequenceComponentGroup.prototype.add.call(this,component);
		}
	},
	/**
	 * protected itemSelected
	 * @param {int} index
	 */
	itemSelected:function(index,suppressEvent){
		if(this._selectionMode == CSequenceComponentGroup.SELECTION_MODE_EXCLUSIVE && this._selectedIndex != -1 && this._selectedIndex != index){
			//deactivate preceding selection
			this.getComponentAt(this._selectedIndex).setChecked(false);
		}
		this._precedingSelectedIndex = this._selectedIndex;
		this._selectedIndex = index;
		if(suppressEvent !== true)
			this.fireSelectionEvent();
	},
	setSelected:function(index,suppressEvent){
		var components = this.getComponents();
		if(index in components){
			components[index].setChecked(true);
			this.itemSelected(index,suppressEvent);
		}
	}
}
/**
 * class CTabbedPaneContent
 * @param {int} tabIndex
 */
var CTabbedPaneContent = function(tabIndex){
	CContainer.call(this);
	//hold the index where it itself was inserted
	this._tabIndex = tabIndex;
}
CTabbedPaneContent.prototype = {
	__proto__:CContainer.prototype,
	setDisplay:function(b){
		if(this._DOMContainerUI){
			this._DOMContainerUI.setStyle(
				"display",
				this._DOMContainerUI.currentStyle("display")=="block" ? "none" : "block"
			);
		}
	},
	/**
	 * @override repaint children components
	 */
	validate:function(){
		var container = this.getDOMContainer();
		if(container){
			container.innerHTML = CContainer.prototype.toHTMLString.call(this);
			//need we notify somebody?
		}
	},
	getIndex:function(){
		return this._tabIndex;
	},
	toHTMLString:function(){
		var childrenHTML = CContainer.prototype.toHTMLString.call(this);
		return "<li"+(this._insets?" style=\"margin:"+this._insets.join(" ")+"\"":"")+" id=\""+this._UIId+"\" class=\"chromeplus-ui-tabbed-pane-tab-content"+(this._cssClassComponents.length?" "+this._cssClassComponents.join(" "):"")+"\">"+childrenHTML+"</li>";
	},
	getDOMContainer:function(){
		return this._DOMContainerUI ? this._DOMContainerUI : (this._DOMContainerUI = document.querySelector("#"+this._UIId));
	},
	initUI:function(){
		CContainer.prototype.initUI.call(this);
		this._DOMContainerUI = document.querySelector("#"+this._UIId);
	}
}
/**
 * class CTabbedPaneTab
 * @param {int} index
 * @param {string} label
 */
var CTabbedPaneTab = function(index,label){
	CContainer.call(this);
	//properties
	this._tabIndex = index;
	this._label = label;
	this._activeCssStyle = "on";
	this._domUI = null;
	this._isActive = false;
	this._isEnabled = true;
	this._selectedEver = false;
}
CTabbedPaneTab.EVENT_IDENTIFIER_ACTION = "C_TABBED_TAB_ACTION";
CTabbedPaneTab.prototype = {
	__proto__:CContainer.prototype,
	addActionListener:function(l){
		this.addPropertyEventListener(CTabbedPaneTab.EVENT_IDENTIFIER_ACTION,l);
	},
	removeActionListener:function(l){
		this.removePropertyEventListener(CTabbedPaneTab.EVENT_IDENTIFIER_ACTION,l);
	},
	fireActionEvent:function(){
		if(!this._isEnabled)
			return;
		this.firePropertyEvent(CTabbedPaneTab.EVENT_IDENTIFIER_ACTION);
	},
	getLabel:function(){
		return this._label;
	},
	setEnabled:function(b){
		Lang.ensureType(b,Lang.BOOLEAN,true);
		if(this._domUI){
			if(b){
				this._domUI.classList.remove("disabled");
			}else{
				this._domUI.classList.add("disabled");
			}
		}
		this._isEnabled = b;
	},
	selectedEver:function(){
		return this._selectedEver;
	},
	setSelectedEver:function(b){
		this._selectedEver = b;
	},
	isActive:function(){
		return this._isActive;
	},
	setUIFocus:function(b){
		if(b){
			this._domUI.classList.add(this._activeCssStyle);
		}else{
			this._domUI.classList.remove(this._activeCssStyle);
		}
	},
	setActive:function(b){
		if(this._domUI){
			if(!this._isEnabled)
				return;
			Lang.ensureType(b,Lang.BOOLEAN,true);
			this.setUIFocus(b);
			this._isActive = b;
		}
	},
	setLabel:function(label){
		if(Lang.ensureType(label,Lang.BOOLEAN) && !label.empty()){
			this._label = label;
			if(this._domUI)
				this._domUI.textContent = this._label;
		}
	},
	getIndex:function(){
		return this._tabIndex;
	},
	setActiveCssStyle:function(style){
		if(Lang.ensureType(style,Lang.STRING) && !style.empty())
			this._activeCssStyle = style;
	},
	toHTMLString:function(){
		var childrenHTML = CContainer.prototype.toHTMLString.call(this);
		return "<li"+(this._insets?" style=\"margin:"+this._insets.join(" ")+"\"":"")+" id=\""+this._UIId+"\" class=\"chromeplus-ui-tabbed-pane-tab"+(this._cssClassComponents.length?" "+this._cssClassComponents.join(" "):"")+"\"><a href=\"javascript:void(0)\"><span>"+this._label+"</span></a>"+childrenHTML+"</li>";
	},
	validate:function(){
		if(this._DOMContainerUI){
			this._DOMContainerUI.innerHTML = this._DOMContainerUI.innerHTML + CContainer.prototype.toHTMLString.call(this);
		}
	},
	getDOMUI:function(){
		return this._domUI;
	},
	initUI:function(){
		CContainer.prototype.initUI.call(this);
		this._domUI = document.querySelector("#"+this._UIId+">a");
		var _this = this;
		this._domUI.addEventListener("click",function(){
			if(_this.isActive())
				return false;
			_this.setActive(true);
			_this.fireActionEvent();
			return true;
		},false);
		this._DOMContainerUI = document.querySelector("#"+this._UIId);
	}
}
/**
 * class CSeparatedTabContentHandler
 */
var CSeparatedTabContentHandler = function(){
	CContainer.call(this);
}
CSeparatedTabContentHandler.prototype = {
	__proto__:CContainer.prototype,
	toHTMLString:function(){
		var childrenHTML = CContainer.prototype.toHTMLString.call(this);
		return "<div class=\"chromeplus-ui-tabbed-pane-tab-contents\"><ul>"+childrenHTML+"</ul></div></div>";
	},
	initUI:function(){
		CContainer.prototype.initUI.call(this);
	}
}
/**
 * class CTabbedPane
 * @param {int} layout mode
 */
var CTabbedPane = function(layoutMode,tabContentHandler){
	CComponent.call(this);
	
	this._layoutMode = layoutMode;
	this._tabContentHandler = tabContentHandler || null;
	
	this._elements = [
		//[tab,tabContent,tabState]
	];
	this._selectedTab = null;
	this._precedingSelectedTab = null;
	
	this._counter = 0;
}
CTabbedPane.EVENT_IDENTIFIER_TAB_SELECTED = "C_TABBED_PANE_TAB_SELECTED";
CTabbedPane.prototype = {
	__proto__:CComponent.prototype,
	toHTMLString:function(){
		var html = ["<div"+(this._insets?" style=\"margin:"+this._insets.join(" ")+"\"":"")+" id=\""+this._UIId+"\" class=\"chromeplus-ui "+(this._layoutMode==CConst.VERTICAL?"chromeplus-ui-layout-vertical":"chromeplus-ui-layout-horizontal")+" chromeplus-ui-tabbed-pane"+(this._cssClassComponents.length?" "+this._cssClassComponents.join(" "):"")+"\">"];
		html.push("<div class=\"chromeplus-ui-tabbed-pane-tabs\"><ul>");
		if(this._tabContentHandler){
			var tabsChildrenHTML = "";
			var handler = new CSeparatedTabContentHandler();
			for(var i=0,len=this._elements.length;i<len;i++){
				tabsChildrenHTML += this._elements[i][0].toHTMLString();
				handler.add(this._elements[i][1]);
			}
			this._tabContentHandler.add(handler);
			html.push(tabsChildrenHTML+"</ul></div></div>");
		}else{
			var tabsChildrenHTML = "", tabContentsChildrenHTML = "";
			for(var i=0,len=this._elements.length;i<len;i++){
				tabsChildrenHTML += this._elements[i][0].toHTMLString();
				tabContentsChildrenHTML += this._elements[i][1].toHTMLString();
			}
			html.push(tabsChildrenHTML+"</ul></div><div class=\"chromeplus-ui-tabbed-pane-tab-contents\"><ul>");
			html.push(tabContentsChildrenHTML+"</ul></div></div>");
		}
		return html.join("");
	},
	addTabChangeListener:function(l){
		this.addPropertyEventListener(CTabbedPane.EVENT_IDENTIFIER_TAB_SELECTED,l);
	},
	removeTabChangeListener:function(){
		this.removePropertyEventListener(CTabbedPane.EVENT_IDENTIFIER_TAB_SELECTED,l);
	},
	fireTabChanged:function(){
		this.firePropertyEvent(CTabbedPane.EVENT_IDENTIFIER_TAB_SELECTED);
	},
	/**
	 * add tab component to this tabbed pane
	 * @param {mixed} a <code>CTabbedPane</code> object or a String holding a label of a <code>CTabbedPaneTab</code>
	 * @param {CTabbedPaneContent} tabContent optional
	 */
	add:function(tab,tabContent){
		if(typeof tab == "string" && !tab.empty()){
			tab = new CTabbedPaneTab(this._counter,tab);
		}else if(tab instanceof CTabbedPaneTab){
			//accept
		}else{
			throw new Error("Inappropriate Argument Type");
		}
		
		if(!(tabContent instanceof CTabbedPaneContent))
			tabContent = new CTabbedPaneContent(this._counter);

		if(!this._inLibraryListener){
			var _this = this;
			this._inLibraryListener = function(){
				//invoke the protected method
				_this.tabSelected(this);
			}
		}
		tab.addActionListener(this._inLibraryListener);
		this._elements.push([tab,tabContent,true]);
	
		this._counter++;
	},
	remove:function(index){
		if((index in this._elements))
			throw new Error("index out of boundary");
		this._elements.splice(index,1);
	},
	setEnabled:function(index,b){
		Lang.ensureType(index,Lang.NUMBER,true);
		Lang.ensureType(b,Lang.BOOLEAN,true);
		
		//the tab should lost focus immediately
		if(!(index in this._elements))
			throw new Error("index out of boundary");
		this._elements[index][2] = b;
		if(b){
			this._elements[index][1].setEnabled(true);
		}else{
			this._elements[index][1].setEnabled(false);
			//focus previous item
			if(this._precedingSelectedIndex != -1)
				this._elements[this._precedingSelectedIndex][1].setEnabled(true);
		}
	},
	insert:function(tab,tabContent,index){
		//insert a tab
	},
	getSelectedIndex:function(){
		return this._selectedTab ? this._selectedTab.getIndex() : -1;
	},
	getPrecedingSelectedIndex:function(){
		return this._precedingSelectedTab ? this._precedingSelectedTab.getIndex() : -1;
	},
	/**
	 * @protected
	 * @param {int} index
	 */
	tabSelected:function(tab,suppressEvent){
		//dispatch event, no updating UI
		if(this._selectedTab && (tab.getIndex() == this._selectedTab.getIndex()))
			return;
		if(this._selectedTab){
			//deactive previous
			this._selectedTab.setActive(false);
			this._elements[this._selectedTab.getIndex()][1].setDisplay(false);
		}
		//tab got focus already
		this._elements[tab.getIndex()][1].setDisplay(true);
		this._precedingSelectedTab = this._selectedTab;
		this._selectedTab = tab;
		if(suppressEvent !== true)
			this.fireTabChanged();//fire event
	},
	setSelected:function(index,suppressEvent){
		if(index in this._elements){
			this._elements[index][0].setActive(true);
			this.tabSelected(this._elements[index][0],suppressEvent);
		}
	},
	getTabAt:function(index){
		if(index in this._elements)
			return this._elements[index][0];
		return null;
	},
	getTabContentAt:function(index){
		if(index in this._elements)
			return this._elements[index][1];
		return null;
	},
	initUI:function(){
		this._elements.forEach(function(item,index){
			item[0].initUI();
			item[1].initUI();
		});
	}
}
/**
 * class CLabel
 * @param {String} label
 */
var CLabel = function(label,affinity){
	Lang.ensureType(label,Lang.STRING,true);
	CComponent.call(this);
	this._label = label;
	this._affinity = Lang.ensureType(affinity,Lang.STRING) ? affinity : "";
	this._domUI = null;
}
CLabel.prototype = {
	__proto__:CComponent.prototype,
	setAffinity:function(aff){
		if(Lang.ensureType(aff,Lang.STRING)){
			this._affinity = aff;
			this._domUI.setAttribute("for",this._affinity);
		}
	},
	setText:function(label){
		if(Lang.ensureType(label,Lang.STRING)){
			this._label = label;
			this._domUI.textContent = this._label;
		}
	},
	getText:function(){
		return this._domUI.textContent.trim();
	},
	toHTMLString:function(){
		if(this._annotation)
			this.addCssClassComponent("chromeplus-ui-what-is-this");
		return "<div"+(this._insets?" style=\"margin:"+this._insets.join(" ")+"\"":"")+" id=\""+this._UIId+"\" class=\"chromeplus-ui chromeplus-ui-label"+(this._cssClassComponents.length?" "+this._cssClassComponents.join(" "):"")+"\"><label for=\""+this._affinity+"\">"+this._label+"</label>"+(this._annotation?this._annotation.toHTMLString():"")+"</div>"; 
	},
	initUI:function(){
		this._domUI = document.querySelector("#"+this._UIId+" > label");
	}
}
/**
 * class CGridLayoutHandler
 * @param {int} columns how many columns
 */
var CGridLayoutHandler = function(columns){
	Lang.ensureType(columns,Lang.NUMBER,true);
	CContainer.call(this);
	
	this._columns = columns;
}
CGridLayoutHandler.prototype = {
	__proto__:CContainer.prototype,
	toHTMLString:function(){
		Lang.assert(this._components.length % this._columns == 0,"elements not enough or more radundancy");
		
		var html = ["<table"+(this._insets?" style=\"margin:"+this._insets.join(" ")+"\"":"")+" id=\""+this._UIId+"\" class=\"chromeplus-ui chromeplus-ui-container chromeplus-ui-grid"+(this._cssClassComponents.length?" "+this._cssClassComponents.join(" "):"")+"\">"];
		html.push("<colgroup>");
		for(var i=0;i<this._columns;i++){
			html.push("<col class=\"column"+(i+1)+"\" />");
		}
		html.push("</colgroup><tbody>");
		
		for(var i=0,len=this._components.length;i<len;i=i+this._columns){
			html.push("<tr>");
			for(var j=i;j<this._columns+i;j++){
				html.push("<td>"+this._components[j].toHTMLString()+"</td>");
			}
			html.push("</tr>");
		}
		html.push("</tbody></table>");
		return html.join("");
	}
}
/**
 * class CRecordSet
 * @param {Object} columnsConfig
 * [
 * 		{column:"id",type:"String",defaultValue:""},
 * 		{column:"name",type:"enum",values:["spade","heart","club","diamond"],defaultValue:0},
 * 		{column:"create_time",type:"Date",defaultValue:"now"}
 * ]
 * @param {Object} commandConfig
 */
var CRecordSet = function(columnsConfig,commandConfig,data){
	CComponent.call(this);
	
	Lang.ensureType(columnsConfig,Lang.ARRAY,true);
	Lang.ensureType(commandConfig,Lang.OBJECT,true);
	
	this._columnConfig = columnsConfig || [];
	this._commandConfig = commandConfig || {};
	this._data = Lang.ensureType(data,Lang.ARRAY) ? data : [];
	this._dispatchReselectionEvent = false;
	
	this._selectedIndex = -1;
	
	this._commandAddUI = null;
	this._commandAddUIEnabled = true;
	this._commandDeleteUI = null;
	this._commandDeleteUIEnabled = true;
	this._commandEditUI = null;
	this._commandEditUIEnabled = true;
	this._tableUI = null;
}
CRecordSet.COMMAND_ADD = 0;
CRecordSet.COMMAND_DELETE = 1;
CRecordSet.COMMAND_EDIT = 2;
CRecordSet.EVENT_IDENTIFIER_COMMAND_ACTIVATE = "C_RECORDSET_COMMAND_ACTIVATE";
CRecordSet.EVENT_IDENTIFIER_ROW_SELECTED = "C_RECORDSET_ROW_SELECTED";
CRecordSet.EVENT_IDENTIFIER_DATA_CHANGED = "C_RECORDSET_DATA_CHANGED";

CRecordSet.LABEL_ADD = "Add";
CRecordSet.LABEL_EDIT = "Edit";
CRecordSet.LABEL_DELETE = "Delete";
CRecordSet.prototype = {
	__proto__:CComponent.prototype,
	toHTMLString:function(){
		if(this._commandConfig.useCommand === false)
			this.addCssClassComponent("chromeplus-not-use-command");
		
		var colgroupHTML = [],theadHTML = [];
		for(var i=0,len=this._columnConfig.length;i<len;i++){
			var col = this._columnConfig[i];
			colgroupHTML.push("<col class=\"column"+(i+1)+"\" />");
			theadHTML.push("<th><strong>"+col.column+"</strong></th>");
		}
		
		var html = ["<div"+(this._insets?" style=\"margin:"+this._insets.join(" ")+"\"":"")+" id=\""+this._UIId+"\" class=\"pseudo-clearfix chromeplus-ui-recordset"+(this._cssClassComponents.length?" "+this._cssClassComponents.join(" "):"")+"\">"];
		html.push("<div class=\"chromeplus-ui-records-container\"><table class=\"chromeplus-ui chromeplus-ui-container chromeplus-ui-grid chromeplus-ui-records\"><caption></caption><colgroup>");
		//push table column group definition
		html.push(colgroupHTML.join(""));
		html.push("</colgroup><thead><tr>");
		//push table head definition
		html.push(theadHTML.join(""));
		html.push("</tr></thead></table><div class=\"chromeplus-ui-records-inner-container\"><table class=\"chromeplus-ui chromeplus-ui-container chromeplus-ui-grid chromeplus-ui-records\"><caption></caption><colgroup>");
		//push table column group definition
		html.push(colgroupHTML.join(""));
		html.push("</colgroup><tbody>");
		//fill with data
		for(var i=0,len=this._data.length;i<len;i++){
			html.push("<tr>");
			for(var j=0,len2=this._data[i].length;j<len2;j++){
				html.push("<td>"+this._data[i][j]+"</td>");
			}
			html.push("</tr>");
		}
		var commandHTML = [];
		if(this._commandConfig.useCommand !== false){
			commandHTML.push("<ul>");
			if(this._commandConfig.useAddCommand !== false)
				commandHTML.push("<li><button class=\"chromeplus-ui-action chromeplus-ui-command-add\" type=\"button\">"+CRecordSet.LABEL_ADD+"</button></li>");
			if(this._commandConfig.useDeleteCommand !== false)
				commandHTML.push("<li><button class=\"chromeplus-ui-action chromeplus-ui-command-delete\" type=\"button\">"+CRecordSet.LABEL_DELETE+"</button></li>");
			if(this._commandConfig.useEditCommand !== false)
				commandHTML.push("<li><button class=\"chromeplus-ui-action chromeplus-ui-command-edit\" type=\"button\">"+CRecordSet.LABEL_EDIT+"</button></li>");
			commandHTML.push("</ul>");
		}
		html.push("</tbody></table></div></div><div class=\"chromeplus-ui-commands\">"+commandHTML.join("")+"</div></div>");
		return html.join("");
	},
	addCommandActivateListener:function(l){
		this.addPropertyEventListener(CRecordSet.EVENT_IDENTIFIER_COMMAND_ACTIVATE,l);
	},
	removeCommandActivateListener:function(l){
		this.removePropertyEventListener(CRecordSet.EVENT_IDENTIFIER_COMMAND_ACTIVATE,l);
	},
	addRowSelectedListener:function(l){
		this.addPropertyEventListener(CRecordSet.EVENT_IDENTIFIER_ROW_SELECTED,l);
	},
	removeRowSelectedListener:function(l){
		this.removePropertyEventListener(CRecordSet.EVENT_IDENTIFIER_ROW_SELECTED,l);
	},
	fireRowSelectedEvent:function(){
		this.firePropertyEvent(CRecordSet.EVENT_IDENTIFIER_ROW_SELECTED,this._selectedIndex);
	},
	addDataChangedListener:function(l){
		this.addPropertyEventListener(CRecordSet.EVENT_IDENTIFIER_DATA_CHANGED,l);
	},
	removeDataChangedListener:function(l){
		this.removePropertyEventListener(CRecordSet.EVENT_IDENTIFIER_DATA_CHANGED,l);
	},
	fireDataChangedEvent:function(i){
		this.firePropertyEvent(CRecordSet.EVENT_IDENTIFIER_DATA_CHANGED,i);
	},
	fireCommandActivateEvent:function(type){
		this.firePropertyEvent(CRecordSet.EVENT_IDENTIFIER_COMMAND_ACTIVATE,type);
	},
	setCommandEnabled:function(commandType,b){
		Lang.ensureType(commandType,Lang.NUMBER,true);
		Lang.ensureType(b,Lang.BOOLEAN,true);
		switch(commandType){
			case CRecordSet.COMMAND_ADD:
				this._commandAddUI.disabled = !b;
				this._commandAddUIEnabled = b;
				break;
			case CRecordSet.COMMAND_DELETE:
				this._commandDeleteUI.disabled = !b;
				this._commandDeleteUIEnabled = b;
				break;
			case CRecordSet.COMMAND_EDIT:
				this._commandEditUI.disabled = !b;
				this._commandEditUIEnabled = b;
				break;
		}
	},
	setDispatchReselectionEvent:function(b){
	    Lang.ensureType(b,Lang.BOOLEAN,true);
	    this._dispatchReselectionEvent = b;
	},
	addData:function(data,index){
		index = Lang.ensureType(index,Lang.NUMBER) ? index : 0;
		if(index < 0 || index>this.getRecordsCount())
			index = 0;
		Lang.ensureType(data,Lang.ARRAY,true);
		
		//deactivate last one
		if(this._selectedIndex != -1){
			this._tableUI.tBodies[0].rows[this._selectedIndex].classList.remove("on");
			this._selectedIndex = -1;
		}
		
		var row = this._tableUI.insertRow(index);
		//update data model
		this._data.splice(index,0,data);
		
		var i=0;
		while(i < this._columnConfig.length){
			row.insertCell(i);
			i++;
		}
		this.setDataAt(row.rowIndex,data);
		var _this = this;
		var editListener = function(){
			if(_this._commandConfig.useCommand !== false && _this._commandConfig.useEditCommand !== false && _this._commandEditUIEnabled)
				_this.fireCommandActivateEvent(CRecordSet.COMMAND_EDIT);
		}
		var selectListener =  function(){
			_this.dataItemSelected(this.rowIndex);
		}
		row.onclick = selectListener;
		row.ondblclick = editListener;
		row.className = "new-row";
		row.scrollIntoView();
		row.addEventListener("webkitAnimationEnd",function(){
			this.classList.remove("new-row");
		},false);
	},
	removeDataAt:function(index){
		if(!this._checkIndex(index))
			return;
		this._tableUI.deleteRow(index);
		//update data model
		this._data.splice(index,1);
		//reset
		this._selectedIndex = -1;
		this.fireDataChangedEvent(index);
	},
	setDataAt:function(index,data){
		if(!this._checkIndex(index))
			return;
		Lang.ensureType(data,Lang.ARRAY,true);
		var row = this._tableUI.tBodies[0].rows[index];
		for(var i=0,len=row.cells.length;i<len;i++){
			if(typeof data[i] != "undefined" && data[i] != null)
				row.cells[i].textContent = data[i];
		}
		//update data model
		this._data[index] = data;
		this.fireDataChangedEvent(index);
	},
	_checkIndex:function(index){
		Lang.ensureType(index,Lang.NUMBER,true);
		return index >= 0 && index < this.getRecordsCount();
	},
	/**
	 * @param {Object} index
	 * @return {Array} 
	 */
	getDataAt:function(index){
		if(!this._checkIndex(index))
			return null;
		var rows = this._tableUI.tBodies[0].rows;
		var data = [],row = rows[index];
		for(var i=0,len=row.cells.length;i<len;i++)
			data.push(row.cells[i].textContent.trim());
		return data;
	},
	getDataModel:function(){
	    return this._data;
	},
	getRecordsCount:function(){
		return this._tableUI.tBodies[0].rows.length;
	},
	getSelectedIndex:function(){
		return this._selectedIndex;
	},
	isRowMarked:function(index){
	       if(index in this._data)
	            return  this._tableUI.tBodies[0].rows[index].getAttribute("distinct") == "true";
	       return false; 
	},
	markRowAsDistinct:function(){
		var from = -1, to = -1;
		if(arguments.length == 1){
			from = to = arguments[0];
		}else if(arguments.length == 2){
			from = arguments[0];
			to = arguments[1];
		}else{
			throw new Error("Illegal Argument Signature");
		}
		
		var rows = this._tableUI.tBodies[0].rows;
		for(var i=from;i<=to;i++){
			rows[i].setAttribute("distinct","true");
		}
	},
	/**
	 * protected dataItemSelected
	 * @param {int} index
	 * @param {boolean} suppressEvent
	 */
	dataItemSelected:function(index,suppressEvent){
		if(!this._dispatchReselectionEvent && (index == this._selectedIndex))
			return;
		var rows = this._tableUI.tBodies[0].rows;
		if(this._selectedIndex != -1){
			rows[this._selectedIndex].classList.remove("on");
		}
		rows[index].classList.add("on");
		this._selectedIndex = index;
		if(suppressEvent !== true){
			//notify idle listeners
			this.fireRowSelectedEvent();
		}
	},
	setSelectedIndex:function(index,suppressEvent){
		this.dataItemSelected(index,suppressEvent);
	},
	initUI:function(){
		this._tableUI = document.querySelectorAll("#"+this._UIId+" table")[1];
		this._commandAddUI = document.querySelector("#"+this._UIId+" .chromeplus-ui-command-add");
		this._commandDeleteUI = document.querySelector("#"+this._UIId+" .chromeplus-ui-command-delete");
		this._commandEditUI = document.querySelector("#"+this._UIId+" .chromeplus-ui-command-edit");
		var _this = this;
		var editListener = function(){
			if(_this._commandConfig.useCommand !== false && _this._commandConfig.useEditCommand !== false && _this._commandEditUIEnabled)
				_this.fireCommandActivateEvent(CRecordSet.COMMAND_EDIT);
		}
		var selectListener =  function(){
			_this.dataItemSelected(this.rowIndex);
		}
		for(var i=0,list=this._tableUI.tBodies[0].rows,len=list.length;i<len;i++){
			list[i].onclick = selectListener;
			list[i].ondblclick = editListener;
		}
		//ADD
		if(this._commandAddUI)
			this._commandAddUI.onclick = function(){
				if(_this._commandConfig.useCommand !== false && _this._commandConfig.useAddCommand !== false && _this._commandAddUIEnabled)
					_this.fireCommandActivateEvent(CRecordSet.COMMAND_ADD);
			}
		//DELETE
		if(this._commandDeleteUI)
			this._commandDeleteUI.onclick = function(){
				if(_this._commandConfig.useCommand !== false && _this._commandConfig.useDeleteCommand !== false && _this._commandDeleteUIEnabled)
					_this.fireCommandActivateEvent(CRecordSet.COMMAND_DELETE);
			}
		//EDIT
		if(this._commandEditUI)
			this._commandEditUI.onclick = editListener;
	}
}
/**
 * class CComboBox
 * @param {String} label 
 * @param {array} items two dimension array
 * [
 * 		[label,value],
 * 		[label,value]
 * ]
 * @param {int} MODE_GRID imply the output will be a grid layout,otherwise, will be flow layout
 */
var CComboBox = function(label,items,layoutMode){
	Lang.ensureType(label,Lang.STRING,true);
	Lang.ensureType(items,Lang.ARRAY,true);
	Lang.ensureType(layoutMode,Lang.NUMBER,true);
	CComponent.call(this);
	this._label = label;
	this._items = items;
	this._layoutMode = layoutMode;
	this._domUI = null;
}
CComboBox.EVENT_IDENTIFIER_SELECTION_CHANGE = "C_COMBO_BOX_SELECTION_CHANGE";
CComboBox.MODE_FLOW = 0;
CComboBox.MODE_GRID = 1;
CComboBox.prototype = {
	__proto__:CComponent.prototype,
	addSelectionChangeListener:function(l){
		if(Lang.ensureType(l,Lang.FUNCTION))
			this.addPropertyEventListener(CComboBox.EVENT_IDENTIFIER_SELECTION_CHANGE,l);
	},
	removeSelectionChangeListener:function(l){
		if(Lang.ensureType(l,Lang.FUNCTION))
			this.removePropertyEventListener(CComboBox.EVENT_IDENTIFIER_SELECTION_CHANGE,l);
	},
	fireSelectionChangeListener:function(){
		this.firePropertyEvent(CComboBox.EVENT_IDENTIFIER_SELECTION_CHANGE);
	},
	/**
	 * protected
	 */
	selectionChanged:function(){
		this.fireSelectionChangeListener();
	},
	getSelectedIndex:function(){
		return this._domUI.selectedIndex;
	},
	setValue:function(v){
	    this._domUI.value = v;
	},
	setSelected:function(i){
	    this._domUI.selectedIndex = i;
	},
	getValue:function(){
		return this._domUI.value;
	},
	setEnabled:function(b){
	    Lang.ensureType(b,Lang.BOOLEAN,true);
	   this._enabled = b; 
	   this._domUI.disabled = !b; 
	},
	insertBefore:function(item,before){
		Lang.ensureType(item,Lang.ARRAY,true);
		if(item.length == 3 && this._domUI.querySelector("optgroup") == null)
			item.shift();
		if(item.length == 3){
			var parent = this._domUI.querySelector("optgroup[label*='"+item[0]+"']");
			if(parent){
				var kid = document.createElement("option");
				kid.text = item[0];
				kid.value = item[1];
				parent.insertBefore(kid,before);
			}
		}else if(item.length == 2){
				var kid = document.createElement("option");
				kid.text = item[0];
				kid.value = item[1];
				this._domUI.insertBefore(kid,before);
		}else{
			throw Error("Illegal argument: "+item);
		}
	},
	insertAfter:function(item,after){
		Lang.ensureType(item,Lang.ARRAY,true);
		if(item.length == 3 && this._domUI.querySelector("optgroup") == null)
			item.shift();
		if(item.length == 3){
			var parent = this._domUI.querySelector("optgroup[label*='"+item[0]+"']");
			if(parent){
				var kid = document.createElement("option");
				kid.text = item[0];
				kid.value = item[1];
				if(after == parent.lastChild)
					parent.appendChild(kid);
				else
					parent.insertBefore(kid,after);
			}
		}else if(item.length == 2){
				var kid = document.createElement("option");
				kid.text = item[0];
				kid.value = item[1];
				if(after == this._domUI.lastChild)
					this._domUI.appendChild(kid);
				else
					this._domUI.insertBefore(kid,after.nextSibling);
		}else{
			throw Error("Illegal argument: "+item);
		}
	},
	remove:function(index){
		if(index in this._domUI.options){
			var kid = this._domUI.options[index];
			kid.parentNode.removeChild(kid);
		}
	},
	getItemCount:function(){
		return this._domUI.options.length;
	},
	toHTMLString:function(){
		if(this._annotation)
			this.addCssClassComponent("chromeplus-ui-what-is-this");
		var childrenHTML = ["<select name=\"dom_"+this._UIId+"\" id=\"dom_"+this._UIId+"\">"];
		if(this._items.length > 0){
			if(this._items[0].length == 3){
				var current = null;
				for(var i=0,len=this._items.length;i<len;i++){
					if(this._items[i][0] != current){
						if(i != 0)
							childrenHTML.push("</optgroup>");
						current = this._items[i][0];
						childrenHTML.push("<optgroup label=\""+current+"\">");
					}
					childrenHTML.push("<option value=\""+this._items[i][2]+"\">"+this._items[i][1]+"</option>");
				}
				childrenHTML.push("</optgroup>");
			}else{
				for (var i = 0, len = this._items.length; i < len; i++) {
					childrenHTML.push("<option value=\""+this._items[i][1]+"\">"+this._items[i][0]+"</option>");
				}
			}
		}
		childrenHTML.push("</select>"+(this._annotation?this._annotation.toHTMLString():""));
		
		if(this._layoutMode == CComboBox.MODE_GRID){
			return "<div"+(this._insets?" style=\"margin:"+this._insets.join(" ")+"\"":"")+" id=\""+this._UIId+"\" class=\"chromeplus-ui chromeplus-layout-grid chromeplus-ui-combobox"+(this._cssClassComponents.length?" "+this._cssClassComponents.join(" "):"")+"\"><table class=\"chromeplus-ui chromeplus-ui-grid\"><colgroup><col class=\"column1\" /><col class=\"column2\" /></colgroup><tbody><tr><td><label for=\"dom_"+this._UIId+"\">"+this._label+"</label></td><td>"+childrenHTML.join("")+"</td></tr></tbody></table></div>";
		}else{
			return "<div"+(this._insets?" style=\"margin:"+this._insets.join(" ")+"\"":"")+" id=\""+this._UIId+"\" class=\"chromeplus-ui chromeplus-layout-grid chromeplus-ui-combobox"+(this._cssClassComponents.length?" "+this._cssClassComponents.join(" "):"")+"\"><label for=\"dom_"+this._UIId+"\">"+this._label+childrenHTML.join("")+"</label></div>";
		}
	},
	initUI:function(){
		this._domUI = document.querySelector("#dom_"+this._UIId);
		var _this = this;
		this._domUI.onchange = function(){
			_this.selectionChanged();
		}
	},
	getDOMUI:function(){
		return this._domUI;
	}
}
/**
 * class CTextField
 * @param {String} label
 * @param {int} layout mode constraint,should be either <code>CTextField.MODE_FLOW</code> or <code>CtextField.MODE_GRID</code>
 * @param {int} accept mode should fall in the value as follows: <ul><li><code>CTextField.MODE_ACCEPT_DATE</code><li><code>CTextField.MODE_ACCEPT_EMAIL</code><li><code>CTextField.MODE_ACCEPT_EVERYTHING</code><li><code>CTextField.MODE_ACCEPT_NUMBER</code><li><code>CTextField.MODE_ACCEPT_STRING</code><li><li><code>CTextField.MODE_ACCEPT_URL</code><li></ul>
 */
var CTextField = function(label,layoutMode,acceptMode){
	Lang.ensureType(label,Lang.STRING,true);
	CContainer.call(this);
	this._label = label;
	this._layoutMode = Lang.ensureType(layoutMode,Lang.NUMBER) ? layoutMode : CTextField.MODE_FLOW;
	this._acceptMode = Lang.ensureType(acceptMode,Lang.NUMBER) ? acceptMode : CTextField.MODE_ACCEPT_EVERYTHING;
	this._value = "";
	
	this._domUI = null;
	this._size = 30;
	this._readonly = false;
	this._enabled = true;
}
CTextField.MODE_ACCEPT_DATE = 0;
CTextField.MODE_ACCEPT_EMAIL = 1;
CTextField.MODE_ACCEPT_EVERYTHING = 2;
CTextField.MODE_ACCEPT_NUMBER = 3;
CTextField.MODE_ACCEPT_STRING = 4;
CTextField.MODE_ACCEPT_URL = 5;

//using by build-in library
CTextField.EVENT_IDENTIFIER_VALIDATION = "C_TEXTFIELD_VALIDATION_EVENT";
CTextField.EVENT_IDENTIFIER_CONFIRM = "C_TEXTFIELD_CONFIRM_EVENT";
CTextField.EVENT_IDENTIFIER_REJECT = "C_TEXTFIELD_REJECT_EVENT";
CTextField.MODE_FLOW = 0;
CTextField.MODE_GRID = 1;

//CTextField.PATTERN_EMAIL = /^([^\s@]+)@([^\s@]+)$/gi;//user@host
//CTextField.PATTERN_URL = /^(((?:[\w-_]+):\/\/)*)([\w-_]+)((?:\.[\w-_])+).*$/gi;

CTextField.prototype = {
	__proto__:CContainer.prototype,
	addValidationListener:function(l){
		this.addPropertyEventListener(CTextField.EVENT_IDENTIFIER_VALIDATION,l);
	},
	removeValidationListener:function(l){
		this.removePropertyEventListener(CTextField.EVENT_IDENTIFIER_VALIDATION,l);
	},
	fireValidationEvent:function(v){
		return this.firePropertyEvent(CTextField.EVENT_IDENTIFIER_VALIDATION,v);
	},
	addConfirmListener:function(l){
		this.addPropertyEventListener(CTextField.EVENT_IDENTIFIER_CONFIRM,l);
	},
	removeConfirmListener:function(l){
		this.removePropertyEventListener(CTextField.EVENT_IDENTIFIER_CONFIRM,l);
	},
	fireConfirmEvent:function(){
		this.firePropertyEvent(CTextField.EVENT_IDENTIFIER_CONFIRM);
	},
	addRejectListener:function(l){
		this.addPropertyEventListener(CTextField.EVENT_IDENTIFIER_REJECT,l);
	},
	removeRejectListener:function(l){
		this.removePropertyEventListener(CTextField.EVENT_IDENTIFIER_REJECT,l);
	},
	fireRejectEvent:function(){
		this.firePropertyEvent(CTextField.EVENT_IDENTIFIER_REJECT);
	},
	/**
	 * protected valueVerified
	 * @param {Object} v
	 * @param {boolean} suppress the event,without notifying listeners
	 */
	valueVerified:function(v,supressEvent){
		if(this.fireValidationEvent(v) && this._ensureValueAccordingToAcceptMode(v)){
			//fire higher-level event
			if(supressEvent !== true)
				this.fireConfirmEvent();
			if(this._domUI)
				this._domUI.removeAttribute("verified");
			return true;
		}else{
			if(this._domUI)
				this._domUI.setAttribute("verified","false");
			if(supressEvent !== true)
				this.fireRejectEvent();
			return false;
		}
	},
	setValue:function(v,supressEvent){
		if(this.valueVerified(v,supressEvent)){
			if(this._domUI){
				this._domUI.value = v;
			}
		}
	},
	longer:function(){
		this._size += 5;
		return this;
	},
	shorter:function(){
		this._size -= 5;
		return this;
	},
	setEnabled:function(b){
		if(Lang.ensureType(b,Lang.BOOLEAN)){
			//super.setEnabled
			CContainer.prototype.setEnabled.call(this,b);
			//self update
			if(this._domUI)
				this._domUI.disabled = !b;
		}
	},
	setReadonly:function(b){
		if(Lang.ensureType(b,Lang.BOOLEAN)){
			this._readonly = b;
			if(this._domUI)
				this._domUI.readonly = b;
		}
	},
	_ensureValueAccordingToAcceptMode:function(value){
		if(Lang.ensureType(value,Lang.STRING) && value.trim().empty()){
			this._value = "";
			return true;
		}
		switch(this._acceptMode){
			case CTextField.MODE_ACCEPT_EVERYTHING:
				this._value = value;
				return true;
			case CTextField.MODE_ACCEPT_DATE:
				var millseconds = Date.parse(value);
				if(!isNaN(millseconds)){
					this._value = millseconds;
					return true;
				}else{
					return false;
				}
			case CTextField.MODE_ACCEPT_EMAIL:
				R.exp.Email.lastIndex = 0;
				if(R.exp.Email.test(value)){
					this._value = value;
					return true;
				}else{
					return false;
				}
			case CTextField.MODE_ACCEPT_NUMBER:
			    R.exp.Number.lastIndex = 0;
				if(R.exp.Number.test(value)){
					this._value = value;
					return true;
				}else{
					return false;
				}
			case CTextField.MODE_ACCEPT_URL:
				R.exp.URL.lastIndex = 0;
				if(R.exp.URL.test(value)){
					this._value = value;
					return true;
				}else{
					return false;
				}
				break;
			case CTextField.MODE_ACCEPT_STRING:
				if(Lang.ensureType(value,Lang.STRING)){
					this._value = value;
					return true;
				}else{
					return false;
				}
			default:
				return false;
		}
	},
	getValue:function(){
		return this._value;
	},
	toHTMLString:function(){
		if(this._annotation)
			this.addCssClassComponent("chromeplus-ui-what-is-this");
		var children = CContainer.prototype.toHTMLString.call(this);
		var childrenHTML = "<input"+(this._readonly?" readonly=\"readonly\"":"")+" spellcheck=\"false\" value=\""+this._value+"\" size=\""+this._size+"\" type=\"text\" name=\"dom_"+this.getUIId()+"\" id=\"dom_"+this.getUIId()+"\">"+(this._annotation?this._annotation.toHTMLString():"")+children;
		if(this._layoutMode == CComboBox.MODE_GRID){
			return "<div"+(this._insets?" style=\"margin:"+this._insets.join(" ")+"\"":"")+" id=\""+this._UIId+"\" class=\"chromeplus-ui chromeplus-ui-textfield"+(this._cssClassComponents.length?" "+this._cssClassComponents.join(" "):"")+"\"><table class=\"chromeplus-ui chromeplus-ui-grid\"><colgroup><col class=\"column1\" /><col class=\"column2\" /></colgroup><tbody><tr><td><label for=\"dom_"+this._UIId+"\">"+this._label+"</label></td><td>"+childrenHTML+"</td></tr></tbody></table></div>";
		}else{
			return "<div"+(this._insets?" style=\"margin:"+this._insets.join(" ")+"\"":"")+" id=\""+this._UIId+"\" class=\"chromeplus-ui chromeplus-ui-textfield"+(this._cssClassComponents.length?" "+this._cssClassComponents.join(" "):"")+"\"><label for=\"dom_"+this._UIId+"\">"+this._label+childrenHTML+"</label></div>";
		}
	},
	getDOMUI:function(){
		return this._domUI;
	},
	initUI:function(){
		CContainer.prototype.initUI.call(this);
		this._domUI = document.querySelector("#dom_"+this._UIId);
		var _this = this;
		var listener = function(e){
			_this.valueVerified(this.value.trim());
		}
		this._domUI.onkeyup = this._domUI.onchange = listener;
		this._domUI.onpaste = function(){
			var me = this;
			setTimeout(function(){
				listener.call(me);
			},5);
		}
	}
}
/**
 * class CTextArea
 * @param {String} label
 * @param {int} layout mode constraint,should be either <code>CTextArea.MODE_FLOW</code> or <code>CTextArea.MODE_GRID</code>
 */
var CTextArea = function(label,layoutMode){
	Lang.ensureType(label,Lang.STRING,true);
	CContainer.call(this);
	this._label = label;
	this._layoutMode = Lang.ensureType(layoutMode,Lang.NUMBER) ? layoutMode : CTextField.MODE_FLOW;

	this._value = "";
	this._domUI = null;
	this._size = null;
	this._readonly = false;
}

//using by build-in library
CTextArea.EVENT_IDENTIFIER_VALIDATION = "C_TEXTFIELD_VALIDATION_EVENT";
CTextArea.EVENT_IDENTIFIER_CONFIRM = "C_TEXTFIELD_CONFIRM";
CTextArea.MODE_FLOW = 0;
CTextArea.MODE_GRID = 1;

CTextArea.prototype = {
	__proto__:CContainer.prototype,
	addValidationListener:function(l){
		this.addPropertyEventListener(CTextField.EVENT_IDENTIFIER_VALIDATION,l);
	},
	removeValidationListener:function(l){
		this.removePropertyEventListener(CTextField.EVENT_IDENTIFIER_VALIDATION,l);
	},
	fireValidationEvent:function(){
		return this.firePropertyEvent(CTextField.EVENT_IDENTIFIER_VALIDATION);
	},
	addConfirmListener:function(l){
		this.addPropertyEventListener(CTextField.EVENT_IDENTIFIER_CONFIRM,l);
	},
	removeConfirmListener:function(l){
		this.removePropertyEventListener(CTextField.EVENT_IDENTIFIER_CONFIRM,l);
	},
	fireConfirmEvent:function(){
		this.firePropertyEvent(CTextField.EVENT_IDENTIFIER_CONFIRM);
	},
	/**
	 * protected valueVerified
	 * @param {Object} v
	 * @param {boolean} suppress the event,without notifying listeners
	 */
	valueVerified:function(v,supressEvent){
		if(this.fireValidationEvent()){
			//fire higher-level event
			if(supressEvent !== true)
				this.fireConfirmEvent();
			this._value = v;
			if(this._domUI)
				this._domUI.removeAttribute("verified");
			return true;
		}else{
			if(this._domUI)
				this._domUI.setAttribute("verified","false");
			return false;
		}
	},
	setSize:function(w,h){
		Lang.ensureType(w,Lang.NUMBER,true);
		Lang.ensureType(h,Lang.NUMBER,true);
		this._size = [w,h];
	},
	setValue:function(v,supressEvent){
		if(this.valueVerified(v,supressEvent)){
			if(this._domUI){
				this._domUI.value = v;
			}
		}
	},
	setEnabled:function(b){
		if(Lang.ensureType(b,Lang.BOOLEAN)){
			//super.setEnabled
			CContainer.prototype.setEnabled.call(this,b);
			//self update
			if(this._domUI)
				this._domUI.disabled = !b;
		}
	},
	setReadonly:function(b){
		if(Lang.ensureType(b,Lang.BOOLEAN))
			this._readonly = b;
	},
	getValue:function(){
		return this._value;
	},
	toHTMLString:function(){
		if(this._annotation)
			this.addCssClassComponent("chromeplus-ui-what-is-this");
		var children = CContainer.prototype.toHTMLString.call(this);
		var childrenHTML = "<textarea"+(this._readonly?" readonly=\"readonly\"":"")+" class=\"chromeplus-ui-textarea-representation\" cols=\"80\" rows=\"10\" spellcheck=\"false\" oncontextmenu=\"return false\" type=\"text\" name=\"dom_"+this.getUIId()+"\""+(this._size?"style=\"width:"+this._size[0]+"px;height:"+this._size[1]+"px\"":"")+" id=\"dom_"+this.getUIId()+"\"></textarea>"+(this._annotation?this._annotation.toHTMLString():"")+children;
		if(this._layoutMode == CComboBox.MODE_GRID){
			return "<div"+(this._insets?" style=\"margin:"+this._insets.join(" ")+"\"":"")+" id=\""+this._UIId+"\" class=\"chromeplus-ui chromeplus-ui-textfield"+(this._cssClassComponents.length?" "+this._cssClassComponents.join(" "):"")+"\"><table class=\"chromeplus-ui chromeplus-ui-grid\"><colgroup><col class=\"column1\" /><col class=\"column2\" /></colgroup><tbody><tr><td><label for=\"dom_"+this._UIId+"\">"+this._label+"</label></td><td>"+childrenHTML+"</td></tr></tbody></table></div>";
		}else{
			return "<div"+(this._insets?" style=\"margin:"+this._insets.join(" ")+"\"":"")+" id=\""+this._UIId+"\" class=\"chromeplus-ui chromeplus-ui-textfield"+(this._cssClassComponents.length?" "+this._cssClassComponents.join(" "):"")+"\"><label for=\"dom_"+this._UIId+"\">"+this._label+childrenHTML+"</label></div>";
		}
	},
	initUI:function(){
		CContainer.prototype.initUI.call(this);
		this._domUI = document.querySelector("#dom_"+this._UIId);
		var _this = this;
		var listener = function(e){
			_this.valueVerified(this.value.trim());
		}
		this._domUI.onkeyup = this._domUI.onchange = listener;
	}
}
/**
 * class CSeparator
 */
var CSeparator = function(){
	CComponent.call(this);
}
CSeparator.prototype = {
	__proto__:CComponent.prototype,
	toHTMLString:function(){
		return "<div class=\"chromeplus-ui-separator\"></div>";
	}
}
/**
 * class CAnnotation it's totally self-contained component
 * @param {String} text
 */
var CAnnotation = function(text){
	Lang.ensureType(text,Lang.STRING,true);
	CComponent.call(this);
	this._text = text;
	this._domUI = null;
}
CAnnotation.prototype = {
	__proto__:CComponent.prototype,
	setText:function(text){
		if(Lang.ensureType(text,Lang.STRING)){
			this._domUI.innerHTML = text;
			this._text = text;
		}
	},
	toHTMLString:function(){
		return "<span"+(this._insets?" style=\"margin:"+this._insets.join(" ")+"\"":"")+" id=\""+this._UIId+"\" class=\"chromeplus-ui chromeplus-ui-annotation"+(this._cssClassComponents.length?" "+this._cssClassComponents.join(" "):"")+"\"><a href=\"javascript:void(0)\" class=\"chromeplus-ui-annotation-trigger\">?</a><span class=\"chromeplus-ui-annotation-content\">"+this._text+"</span></span>";
	}
}
/**
 * class CAction
 * @param {String} label
 * @param {int} mode
 */
var CAction = function(label,mode){
	Lang.ensureType(label,Lang.STRING,true);
	CComponent.call(this);
	//property
	this._loadingStyle = "loading";
	this._isLoading = false;
	this._label = label;
	this._mode = Lang.ensureType(mode,Lang.NUMBER) ? mode : CAction.MODE_LAYOUT_INLINE;
	this._domUI = null;
}
CAction.EVENT_IDENTIFIER_ACTION = "C_ACTION_ACTION";
CAction.MODE_LAYOUT_INLINE = 0;
CAction.MODE_LAYOUT_BLOCK = 1;
CAction.prototype = {
	__proto__:CComponent.prototype,
	toHTMLString:function(){
		if(this._mode == CAction.MODE_LAYOUT_BLOCK){
			return "<div"+(this._insets?" style=\"margin:"+this._insets.join(" ")+"\"":"")+" id=\""+this._UIId+"\" class=\"chromeplus-ui chromeplus-ui-action-component"+(this._cssClassComponents.length?" "+this._cssClassComponents.join(" "):"")+"\"><a class=\"chromeplus-ui-action chromeplus-layout-block\" href=\"javascript:void(0)\"><span>"+this._label+"</span></a></div>";
		}else{
			return "<a"+(this._insets?" style=\"margin:"+this._insets.join(" ")+"\"":"")+" id=\""+this._UIId+"\" class=\"chromeplus-layout-inline chromeplus-ui-action"+(this._cssClassComponents.length?" "+this._cssClassComponents.join(" "):"")+"\" href=\"javascript:void(0)\"><span>"+this._label+"</span></a>";
		}
	},
	setText:function(text){
		Lang.ensureType(text,Lang.STRING,true);
		this._label = text;
		if(this._domUI)
			this._domUI.querySelector("span").textContent = text;
	},
	getText:function(){
		return this._label;
	},
	setEnabled:function(b){
		Lang.ensureType(b,Lang.BOOLEAN,true);
		this._enabled = b;
		if(this._domUI){
			if(this._enabled)
				this._domUI.classList.remove("disabled");
			else
				this._domUI.classList.add("disabled");
		}
	},
	getDOMUI:function(){
		return this._domUI;
	},
	addActionListener:function(l){
		this.addPropertyEventListener(CAction.EVENT_IDENTIFIER_ACTION,l);
	},
	removeActionListener:function(l){
		this.removePropertyEventListener(CAction.EVENT_IDENTIFIER_ACTION,l);
	},
	isLoading:function(){
		return this._isLoading;
	},
	setLoading:function(b){
		Lang.ensureType(b,Lang.BOOLEAN,true);
		this._isLoading = b;
		this.setEnabled(!b);
		if(this._domUI)
			if(b)
				this._domUI.classList.add(this._loadingStyle);
			else
				this._domUI.classList.remove(this._loadingStyle);
	},
	fireActionEvent:function(){
		this.firePropertyEvent(CAction.EVENT_IDENTIFIER_ACTION);
	},
	initUI:function(){
		this._domUI = (this._mode==CAction.MODE_LAYOUT_BLOCK) ? document.querySelector("#"+this._UIId+" >.chromeplus-ui-action") : document.querySelector("#"+this._UIId);
		var _this = this;
		this._domUI.addEventListener("click",function(){
			if(!_this._enabled || _this._isLoading)
				return;
			_this.fireActionEvent();
		},false);
	}
}
/***
 * PersistenceResolver
 */
var PersistenceResolver = function(){
    //the handler retain prefs data
    this._persistenceData = null;
    //callbacks to communicate with C++
    this._nativeCodeOutgoingCallback = null;
}
PersistenceResolver.prototype = {
    /*update:function(key,value){
        this._persistenceData[key] = value;
        console.log("update: "+key);
    },*/
    persist:function(key){
        const FLAG_END = "--sending_data_end-flag--";
        var outgoing = [];
        for(var item in this._persistenceData)
            outgoing.push(
                item,
                String(this._persistenceData[item])
            );
        outgoing.push(FLAG_END);
        console.log("before sending:"+this._nativeCodeOutgoingCallback);
        chrome.send(this._nativeCodeOutgoingCallback,outgoing);
    },
    setOutgoingCallback:function(callback){
        this._nativeCodeOutgoingCallback = callback;
    },
    setData:function(data){
        this._persistenceData = data;
    },
    getDataItemAsBoolean:function(key,defaultValue){
        const DEFAULT_VALUE = false;
        if(!Lang.ensureType(defaultValue,Lang.BOOLEAN))
            defaultValue = DEFAULT_VALUE;
        if(Lang.ensureType(key,Lang.UNDEFINED) || !Lang.ensureType(this._persistenceData[key],Lang.BOOLEAN))
            return defaultValue;
        return this._persistenceData[key];
    },
    getDataItemAsInt:function(key,defaultValue){
        const DEFAULT_VALUE = 0;
        if(!Lang.ensureType(defaultValue,Lang.NUMBER))
            defaultValue = DEFAULT_VALUE;
        if(Lang.ensureType(key,Lang.UNDEFINED) || Lang.ensureType(this._persistenceData[key],Lang.UNDEFINED))
            return defaultValue;
        var value = parseInt(this._persistenceData[key],10);
        return isNaN(value) ? defaultValue : value;
    },
    getDataItemAsString:function(key,defaultValue){
        const DEFAULT_VALUE = "";
        if(Lang.ensureType(defaultValue,Lang.UNDEFINED))
            defaultValue = DEFAULT_VALUE;
        if(Lang.ensureType(key,Lang.UNDEFINED) || Lang.ensureType(this._persistenceData[key],Lang.UNDEFINED))
            return defaultValue;
        return String(this._persistenceData[key]);
    }
}
/**
 * class CToast
 */
var CToast = function(){
	CWindow.call(this);
	this._domUI = null;
	this._thread = null;
}
CToast.prototype = {
	__proto__:CWindow.prototype,
	toHTMLString:function(){
		return "<div id=\""+this._UIId+"\" class=\"chromeplus-ui-toast disappear\"></div>";
	},
	setMessage:function(msg){
		this._domUI.innerHTML = msg;
		this.setVisible(true);
		return this;
	},
	setVisible:function(b){
		Lang.ensureType(b,Lang.BOOLEAN,true);
		if(b){
			this._domUI.classList.remove("disappear");
			this._alignCenter();
			if(this._thread.isAlive())
				this._thread.interrupt();
			this._thread.start();
		}else{
			this._domUI.classList.add("disappear");
		}
		this.fireComponentEvent();
	},
	setLocation:function(left,top){
		Lang.ensureType(left,Lang.NUMBER,true);
		Lang.ensureType(top,Lang.NUMBER,true);
		this._domUI.setStyle("left",left+"px");
		this._domUI.setStyle("top",top+"px");
		this._location = [left,top];
		return this;
	},
	_alignCenter:function(){
		this._domUI.setStyle("left",(document.documentElement.clientWidth-this._domUI.offsetWidth)/2+"px");
	},
	setSize:function(){
		//do nothing
	},
	initUI:function(){
		this._domUI = document.querySelector("#"+this._UIId);
		var _this = this;
		this._domUI.addEventListener("webkitTransitionEnd",function(){
			if(this.classList.contains("disappear")){
				_this._isVisible = false;
				this.setStyle("left","-9999px");
			}else{
				_this._isVisible = true;
			}
		},false);
		window.addEventListener("resize",function(){
			_this._alignCenter();
		},false);
		
		this._thread = new CTimer(5000,function(){
			_this.setVisible(false);
		});
		this._domUI.addEventListener("mouseover",function(){
			if(_this._thread.isAlive())
				_this._thread.interrupt();
		},false);
		this._domUI.addEventListener("mouseout",function(){
			if(!_this._thread.isAlive())
				_this._thread.start();
		},false);
	}
}
/**
 * RemoteContentResolver
 */ 
var RemoteContentResolver = function(uri,timeout){
    Lang.ensureType(uri,Lang.STRING,true);
    this._xhr = null; 
    this._uri = uri;
    this._method =  RemoteContentResolver.METHOD_GET;
    this._timeout =  Lang.ensureType(timeout,Lang.NUMBER) ? timeout : RemoteContentResolver.TIMEOUT;
    this._deamon = null;
    this._data = null;
    this._aborted = false;
   
    this._responsed = false;    
}
RemoteContentResolver.TIMEOUT = 30 * 1000;
RemoteContentResolver.METHOD_GET = "GET";
RemoteContentResolver.METHOD_POST = "POST";
RemoteContentResolver.prototype = {
   resolve:function(callback){
       if(Lang.ensureType(callback.onBeforeRequest,Lang.FUNCTION))
           callback.onBeforeRequest.call(this);
       this._xhr = new XMLHttpRequest();
       this._aborted = false; 
       var _this = this;
       this._xhr.onreadystatechange = function(){
            if(_this._xhr.readyState == 4){
                if(_this._xhr.status == 200){
                    if(_this._aborted) {
                        if(Lang.ensureType(callback.onFailure,Lang.FUNCTION)) 
                                callback.onFailure.call(_this);
                        return;
                    }
                    if(Lang.ensureType(callback.onSuccess,Lang.FUNCTION)) 
                        callback.onSuccess.call(_this,_this._xhr.responseText);
                }else{
                    if(Lang.ensureType(callback.onFailure,Lang.FUNCTION)) 
                            callback.onFailure.call(_this);
                }
               _this._responsed = true;
               if( _this._deamon && _this._deamon.isAlive()){
                    _this._deamon.interrupt();
                    _this._deamon = null;
               }   
            }
       }
       this._xhr.open(this._method,this._uri,true);  
       this._xhr.send(this._data);
       this._deamon = new CTimer(this._timeout,function(){
            if(!_this._responsed){
                   _this._responsed = true;
                   _this._xhr.abort();
                  console.log("interrupted by timing-out");  
            }
       });
       this._deamon.start();
       if(Lang.ensureType(callback.onRequest,Lang.FUNCTION)) 
                callback.onRequest.call(this);
   },
   setData:function(data){
        this._data = data;
   },
   interrupt:function(){
       if( this._deamon && this._deamon.isAlive()){
            this._deamon.interrupt();
            this._deamon = null;
       }
       this._xhr.abort();
       this._aborted = true;
   },  
   setMethod:function(m){
        switch(m) {
            case RemoteContentResolver.METHOD_GET:
            case RemoteContentResolver.METHOD_POST:
                    this._method = m;
                    break;
             default:
        } 
    }     
}
/**
 * AdBlockRuleFilter
 */
var AdBlockRuleFilter = function(rule){
   this._elementsData = null;
   Lang.ensureType(rule,Lang.STRING,true/*raise exception*/); 
   this._rawRules = rule.trim();
   //validate rule
   this._filter(); 
}
AdBlockRuleFilter.RULE_DELIMITER = "\n";
AdBlockRuleFilter.MIN_CONTENT_LENGTH = 5000;
AdBlockRuleFilter.prototype = {
    getAllLegalRules:function(){
         return this._elementsData.length ? this._elementsData.join(AdBlockRuleFilter.RULE_DELIMITER) : null;
    },
    _filter:function(){
        this._elementsData = [];
        if(this._rawRules.length < AdBlockRuleFilter.MIN_CONTENT_LENGTH) 
            return;
        var list = this._rawRules.split(AdBlockRuleFilter.RULE_DELIMITER);
        for(var i=0,len=list.length;i<len;i++){
            var rule = list[i].trim();
            if (rule && rule[0] != '!' && rule[0] != '[' && rule[0] != '@' && !(rule[0] == '/' && rule[rule.length - 1] == '/')){
                if (rule.indexOf('#') != -1) {
                    this._elementsData.push(rule);
                }
                //converting rule
            }
        }  
    }   
}
/**
 * CKeyStroke
 */
var CKeyStroke = function(){
    this._event = null;
}
CKeyStroke.DOM_VK_CANCEL = 3;
CKeyStroke.DOM_VK_HELP = 6;
CKeyStroke.DOM_VK_BACK_SPACE = 8;
CKeyStroke.DOM_VK_TAB = 9;
CKeyStroke.DOM_VK_CLEAR = 12;
CKeyStroke.DOM_VK_RETURN = 13;
CKeyStroke.DOM_VK_ENTER = 14;
CKeyStroke.DOM_VK_SHIFT = 16;
CKeyStroke.DOM_VK_CONTROL = 17;
CKeyStroke.DOM_VK_ALT = 18;
CKeyStroke.DOM_VK_PAUSE = 19;
CKeyStroke.DOM_VK_CAPS_LOCK = 20;
CKeyStroke.DOM_VK_ESCAPE = 27;
CKeyStroke.DOM_VK_SPACE = 32;
CKeyStroke.DOM_VK_PAGE_UP = 33;
CKeyStroke.DOM_VK_PAGE_DOWN = 34;
CKeyStroke.DOM_VK_END = 35;
CKeyStroke.DOM_VK_HOME = 36;
CKeyStroke.DOM_VK_LEFT = 37;
CKeyStroke.DOM_VK_UP = 38;
CKeyStroke.DOM_VK_RIGHT = 39;
CKeyStroke.DOM_VK_DOWN = 40;
CKeyStroke.DOM_VK_SELECT = 41;
CKeyStroke.DOM_VK_PRINT = 42;
CKeyStroke.DOM_VK_PRINTSCREEN = 44;
CKeyStroke.DOM_VK_INSERT = 45;
CKeyStroke.DOM_VK_DELETE = 46;
CKeyStroke.DOM_VK_HELP = 47;
CKeyStroke.DOM_VK_0 = 48;
CKeyStroke.DOM_VK_1 = 49;
CKeyStroke.DOM_VK_2 = 50;
CKeyStroke.DOM_VK_3 = 51;
CKeyStroke.DOM_VK_4 = 52;
CKeyStroke.DOM_VK_5 = 53;
CKeyStroke.DOM_VK_6 = 54;
CKeyStroke.DOM_VK_7 = 55;
CKeyStroke.DOM_VK_8 = 56;
CKeyStroke.DOM_VK_9 = 57;
CKeyStroke.DOM_VK_SEMICOLON = 59;
CKeyStroke.DOM_VK_EQUALS = 61;
CKeyStroke.DOM_VK_A = 65;
CKeyStroke.DOM_VK_B = 66;
CKeyStroke.DOM_VK_C = 67;
CKeyStroke.DOM_VK_D = 68;
CKeyStroke.DOM_VK_E = 69;
CKeyStroke.DOM_VK_F = 70;
CKeyStroke.DOM_VK_G = 71;
CKeyStroke.DOM_VK_H = 72;
CKeyStroke.DOM_VK_I = 73;
CKeyStroke.DOM_VK_J = 74;
CKeyStroke.DOM_VK_K = 75;
CKeyStroke.DOM_VK_L = 76;
CKeyStroke.DOM_VK_M = 77;
CKeyStroke.DOM_VK_N = 78;
CKeyStroke.DOM_VK_O = 79;
CKeyStroke.DOM_VK_P = 80;
CKeyStroke.DOM_VK_Q = 81;
CKeyStroke.DOM_VK_R = 82;
CKeyStroke.DOM_VK_S = 83;
CKeyStroke.DOM_VK_T = 84;
CKeyStroke.DOM_VK_U = 85
CKeyStroke.DOM_VK_V = 86;
CKeyStroke.DOM_VK_W = 87;
CKeyStroke.DOM_VK_X = 88;
CKeyStroke.DOM_VK_Y = 89;
CKeyStroke.DOM_VK_Z = 90;
CKeyStroke.DOM_VK_CONTEXT_MENU = 93;
CKeyStroke.DOM_VK_NUMPAD0 = 96;
CKeyStroke.DOM_VK_NUMPAD1 = 97;
CKeyStroke.DOM_VK_NUMPAD2 = 98;
CKeyStroke.DOM_VK_NUMPAD3 = 99;
CKeyStroke.DOM_VK_NUMPAD4 = 100;
CKeyStroke.DOM_VK_NUMPAD5 = 101;
CKeyStroke.DOM_VK_NUMPAD6 = 102;
CKeyStroke.DOM_VK_NUMPAD7 = 103;
CKeyStroke.DOM_VK_NUMPAD8 = 104;
CKeyStroke.DOM_VK_NUMPAD9 = 105;
CKeyStroke.DOM_VK_MULTIPLY = 106;
CKeyStroke.DOM_VK_ADD = 107;
CKeyStroke.DOM_VK_SEPARATOR = 108;
CKeyStroke.DOM_VK_SUBTRACT = 109;
CKeyStroke.DOM_VK_DECIMAL = 110;
CKeyStroke.DOM_VK_DIVIDE = 111;
CKeyStroke.DOM_VK_F1 = 112;
CKeyStroke.DOM_VK_F2 = 113;
CKeyStroke.DOM_VK_F3 = 114;
CKeyStroke.DOM_VK_F4 = 115;
CKeyStroke.DOM_VK_F5 = 116;
CKeyStroke.DOM_VK_F6 = 117;
CKeyStroke.DOM_VK_F7 = 118;
CKeyStroke.DOM_VK_F8 = 119;
CKeyStroke.DOM_VK_F9 = 120;
CKeyStroke.DOM_VK_F10 = 121;
CKeyStroke.DOM_VK_F11 = 122;
CKeyStroke.DOM_VK_F12 = 123;
CKeyStroke.DOM_VK_F13 = 124;
CKeyStroke.DOM_VK_F14 = 125;
CKeyStroke.DOM_VK_F15 = 126;
CKeyStroke.DOM_VK_F16 = 127;
CKeyStroke.DOM_VK_F17 = 128;
CKeyStroke.DOM_VK_F18 = 129;
CKeyStroke.DOM_VK_F19 = 130;
CKeyStroke.DOM_VK_F20 = 131;
CKeyStroke.DOM_VK_F21 = 132;
CKeyStroke.DOM_VK_F22 = 133;
CKeyStroke.DOM_VK_F23 = 134;
CKeyStroke.DOM_VK_F24 = 135;
CKeyStroke.DOM_VK_NUM_LOCK = 144;
CKeyStroke.DOM_VK_SCROLL_LOCK = 145;
CKeyStroke.DOM_VK_ADD_EX = 187;
CKeyStroke.DOM_VK_COMMA = 188;
CKeyStroke.DOM_VK_SUBTRACT_EX = 189;
CKeyStroke.DOM_VK_PERIOD = 190;
CKeyStroke.DOM_VK_SLASH = 191;
CKeyStroke.DOM_VK_BACK_QUOTE = 192;
CKeyStroke.DOM_VK_OPEN_BRACKET = 219;
CKeyStroke.DOM_VK_BACK_SLASH = 220;
CKeyStroke.DOM_VK_CLOSE_BRACKET = 221;
CKeyStroke.DOM_VK_QUOTE = 222;
CKeyStroke.DOM_VK_META = 224;

CKeyStroke.KEY_DELIMITER = " + ";
CKeyStroke.KEY_TEXT_CONTROL = "Ctrl";
CKeyStroke.KEY_TEXT_ALT = "Alt";
CKeyStroke.KEY_TEXT_SHIFT = "Shift";

CKeyStroke.prototype = {
  isAltDown:function(){
       return this._event.altKey;
  },
  setDOMEvent:function(e){
       if(!this._event)
           this._event = {};
       //inline copy
       this._event.keyCode = e.keyCode;
       this._event.charCode = e.charCode;
       this._event.altKey = e.altKey;
       this._event.ctrlKey = e.ctrlKey;
       this._event.shiftKey = e.shiftKey;
       this._event.metaKey = e.metaKey;
  }, 
  isCtrlDown:function(){
       return this._event.ctrlKey;
  },
  isShiftDown:function(){
       return this._event.shiftKey;
  },
  isMetaDown:function(){
       return this._event.metaKey;
  },
  acceptable:function(){
    return !this.isModifiersOnly() && (this._keyTextFromCode() != null);
  },
  isModifiersOnly:function(){
       switch(this._event.keyCode){
            case CKeyStroke.DOM_VK_CONTROL:
            case CKeyStroke.DOM_VK_SHIFT:
            case CKeyStroke.DOM_VK_ALT:
            //case CKeyStroke.DOM_VK_META:
                return true;
            default:
                return false; 
       } 
   },  
   getKeyModifiersText:function(){
       var modifiers = [];
       if(this.isCtrlDown())
            modifiers.push(CKeyStroke.KEY_TEXT_CONTROL);
       if(this.isAltDown())
            modifiers.push(CKeyStroke.KEY_TEXT_ALT);
       if(this.isShiftDown())
            modifiers.push(CKeyStroke.KEY_TEXT_SHIFT);
       return modifiers;
   }, 
   getKeyText:function(){
        if(this.isModifiersOnly()) 
            return null;
        return this._keyTextFromCode();
   },
   _keyTextFromCode:function(){
       switch(this._event.keyCode){
            case CKeyStroke.DOM_VK_BACK_SPACE:
                return "Backspace";
            case CKeyStroke.DOM_VK_TAB:
                return "Tab";
            case CKeyStroke.DOM_VK_CLEAR:
                return "Clear";
            case CKeyStroke.DOM_VK_RETURN:
                return "Enter";
            case CKeyStroke.DOM_VK_PAUSE:
                return "Pause";
            case CKeyStroke.DOM_VK_CAPS_LOCK:
                return "CapsLock";
            case CKeyStroke.DOM_VK_ESCAPE:
                return "Esc";
            case CKeyStroke.DOM_VK_SPACE:
                return "Space";
            case CKeyStroke.DOM_VK_PAGE_UP:
                return "PgUp";
            case CKeyStroke.DOM_VK_PAGE_DOWN:
                return "PgDn";
            case CKeyStroke.DOM_VK_END:
                return "End";
            case CKeyStroke.DOM_VK_HOME:
                return "Home";
            case CKeyStroke.DOM_VK_LEFT:
                return "Left";
            case CKeyStroke.DOM_VK_UP:
                return "Up";
            case CKeyStroke.DOM_VK_RIGHT:
                return "Right";
            case CKeyStroke.DOM_VK_DOWN:
                return "Down";
            case CKeyStroke.DOM_VK_SELECT:
                return "Select";
            case CKeyStroke.DOM_VK_PRINT:
                return "Print";
            case CKeyStroke.DOM_VK_INSERT:
                return "Insert";
            case CKeyStroke.DOM_VK_DELETE:
                return "Delete";
            case CKeyStroke.DOM_VK_HELP:
                return "Help";
            case CKeyStroke.DOM_VK_0:
            case CKeyStroke.DOM_VK_NUMPAD0:
                return "0";
            case CKeyStroke.DOM_VK_1:
            case CKeyStroke.DOM_VK_NUMPAD1:
                return "1";
            case CKeyStroke.DOM_VK_2:
            case CKeyStroke.DOM_VK_NUMPAD2:
                return "2";
            case CKeyStroke.DOM_VK_3:
            case CKeyStroke.DOM_VK_NUMPAD3:
                return "3";
            case CKeyStroke.DOM_VK_4:
            case CKeyStroke.DOM_VK_NUMPAD4:
                return "4";
            case CKeyStroke.DOM_VK_5:
            case CKeyStroke.DOM_VK_NUMPAD5:
                return "5";
            case CKeyStroke.DOM_VK_6:
            case CKeyStroke.DOM_VK_NUMPAD6:
                return "6";
            case CKeyStroke.DOM_VK_7:
            case CKeyStroke.DOM_VK_NUMPAD7:
                return "7";
            case CKeyStroke.DOM_VK_8:
            case CKeyStroke.DOM_VK_NUMPAD8:
                return "8";
            case CKeyStroke.DOM_VK_9:
            case CKeyStroke.DOM_VK_NUMPAD9:
                return "9";
            case CKeyStroke.DOM_VK_A:
                return "A";
            case CKeyStroke.DOM_VK_B:
                return "B";
            case CKeyStroke.DOM_VK_C:
                return "C";
            case CKeyStroke.DOM_VK_D:
                return "D";
            case CKeyStroke.DOM_VK_E:
                return "E";
            case CKeyStroke.DOM_VK_F:
                return "F";
            case CKeyStroke.DOM_VK_G:
                return "G";
            case CKeyStroke.DOM_VK_H:
                return "H";
            case CKeyStroke.DOM_VK_I:
                return "I";
            case CKeyStroke.DOM_VK_J:
                return "J";
            case CKeyStroke.DOM_VK_K:
                return "K";
            case CKeyStroke.DOM_VK_L:
                return "L";
            case CKeyStroke.DOM_VK_M:
                return "M";
            case CKeyStroke.DOM_VK_N:
                return "N";
            case CKeyStroke.DOM_VK_O:
                return "O";
            case CKeyStroke.DOM_VK_P:
                return "P";
            case CKeyStroke.DOM_VK_Q:
                return "Q";
            case CKeyStroke.DOM_VK_R:
                return "R";
            case CKeyStroke.DOM_VK_S:
                return "S";
            case CKeyStroke.DOM_VK_T:
                return "T";
            case CKeyStroke.DOM_VK_U:
                return "U";
            case CKeyStroke.DOM_VK_V:
                return "V";
            case CKeyStroke.DOM_VK_W:
                return "W";
            case CKeyStroke.DOM_VK_X:
                return "X";
            case CKeyStroke.DOM_VK_Y:
                return "Y";
            case CKeyStroke.DOM_VK_Z:
                return "Z";
            case CKeyStroke.DOM_VK_MULTIPLY:
                return "*";
            case CKeyStroke.DOM_VK_ADD:
            case CKeyStroke.DOM_VK_ADD_EX:
                return "+";
            case CKeyStroke.DOM_VK_SUBTRACT:
            case CKeyStroke.DOM_VK_SUBTRACT_EX:
                return "-";
            case CKeyStroke.DOM_VK_DIVIDE:
                return "/";
            case CKeyStroke.DOM_VK_F1:
                return "F1";
            case CKeyStroke.DOM_VK_F2:
                return "F2";
            case CKeyStroke.DOM_VK_F3:
                return "F3";
            case CKeyStroke.DOM_VK_F4:
                return "F4";
            case CKeyStroke.DOM_VK_F5:
                return "F5";
            case CKeyStroke.DOM_VK_F6:
                return "F6";
            case CKeyStroke.DOM_VK_F7:
                return "F7";
            case CKeyStroke.DOM_VK_F8:
                return "F8";
            case CKeyStroke.DOM_VK_F9:
                return "F9";
            case CKeyStroke.DOM_VK_F10:
                return "F10";
            case CKeyStroke.DOM_VK_F11:
                return "F11";
            case CKeyStroke.DOM_VK_F12:
                return "F12";
            case CKeyStroke.DOM_VK_NUM_LOCK:
                return "NumLock";
            case CKeyStroke.DOM_VK_SCROLL_LOCK:
                return "ScrLk";
            default:
                return null;
       } 
   }, 
   getText:function(){
        var modifiers =  this.getKeyModifiersText(), text = this.getKeyText();
        if(text)
            modifiers.push(text);
        return modifiers.join(CKeyStroke.KEY_DELIMITER);
   }  
}
/**
 * CDropdownMenu
 */
var CDropdownMenu = function(listModel){
    CWindow.call(this);
    Lang.ensureType(listModel,Lang.ARRAY,true);
    this._listModel = listModel;
    this._domUI = null;
    this._listUI = null;
}
CDropdownMenu.EVENT_IDENTIFIER_ITEM_SELECTED = "C_DROPDOWN_ITEM_SELECTED";
CDropdownMenu.prototype = {
    __proto__:CWindow.prototype,
	addItemSelectedListener:function(l){
		this.addPropertyEventListener(CDropdownMenu.EVENT_IDENTIFIER_ITEM_SELECTED,l);
	},
	removeItemSelectedListener:function(){
		this.removePropertyEventListener(CDropdownMenu.EVENT_IDENTIFIER_ITEM_SELECTED,l);
	},
	fireItemSelectedEvent:function(index){
		this.firePropertyEvent(CDropdownMenu.EVENT_IDENTIFIER_ITEM_SELECTED,index);
	},
	setSize:function(w,h){
		Lang.ensureType(w,Lang.NUMBER,true);
		Lang.ensureType(h,Lang.NUMBER,true);
		this._domUI.setStyle("width",w+"px");
		this._domUI.setStyle("height",h+"px");
		this._size = [w,h];
	},
	setVisible:function(b){
		Lang.ensureType(b,Lang.BOOLEAN,true);
		if(this._size[0] == 0)
			this._size = [this._domUI.offsetWidth,this._domUI.offsetHeight];
		if(b){
			this._domUI.classList.remove("dropdown-disappear");
		}else{
			this._domUI.classList.add("dropdown-disappear");
		}
	},
	setLocation:function(left,top){
		Lang.ensureType(left,Lang.NUMBER,true);
		Lang.ensureType(top,Lang.NUMBER,true);
		this._domUI.setStyle("left",left+"px");
		this._domUI.setStyle("top",top+"px");
		this._location = [left,top];
	},
	setZIndex:function(z){
		Lang.ensureType(z,Lang.NUMBER,true);
		this._domUI.setStyle("z-index",z);
		this._z = z;
	},
	toHTMLString:function(){
	    var children = [];
	    for(var i=0,len=this._listModel.length;i<len;i++){
	        children.push("<li><a index=\""+i+"\" href=\"javascript:void(0)\"><span>"+this._listModel[i]+"</span></a></li>");
	    }
		return "<div id=\""+this._UIId+"\" class=\"chromeplus-ui chromeplus-ui-dropdown dropdown-disappear"+(this._cssClassComponents.length?" "+this._cssClassComponents.join(" "):"")+"\"><ul>"+children.join("")+"</ul></div>";
	},
	getDOMUI:function(){
		return this._domUI;
	},
	itemSelected:function(index){
	    this.fireItemSelectedEvent(index);
	},
	getItemAt:function(index){
	    return this._listUI[index];
	},
	initUI:function(){
		var _this = this;
		this._domUI = document.querySelector("#"+this._UIId);
		this._domUI.addEventListener("webkitTransitionEnd",function(){
			_this._isVisible = !this.classList.contains("dropdown-disappear");
			_this.fireComponentEvent();
		},false);
		var actionListener = function(){
		    _this.itemSelected(
		        parseInt(this.getAttribute("index"),10)
		    );
		}
		for(var i=0,list=this._domUI.querySelectorAll("a"),len=list.length;i<len;i++){
		    list[i].onclick = actionListener;
		}
		this._listUI = list;
	}
}
/**
 * CLinearLayout
 */
var CLinearLayout = function(){
    CContainer.call(this);
}
CLinearLayout.prototype = {
    __proto__:CContainer.prototype,
    validate:function(){
		this._DOMContainerUI = document.querySelector("#"+this._UIId);
		this._DOMContainerUI.innerHTML = CContainer.prototype.toHTMLString.call(this);
	},
    toHTMLString:function(){
       var children = CContainer.prototype.toHTMLString.call(this);
       return "<div"+(this._insets?" style=\"margin:"+this._insets.join(" ")+"\"":"")+" id=\""+this._UIId+"\" class=\"chromeplus-ui chromeplus-ui-container relative-box chromeplus-ui-linear-layout pseudo-clearfix"+(this._cssClassComponents.length?" "+this._cssClassComponents.join(" "):"")+"\">"+children+"</div>";
    }
}
/**
 * CSplitButton
 */
var CSplitButton = function(icon){
    CComponent.call(this);
    this._iconCSS = icon;
    this._domUI = null;
}
CSplitButton.EVENT_IDENTIFIER_ACTION = "C_SPLIT_BUTTON_ACTION_EVENT";
CSplitButton.EVENT_IDENTIFIER_DROPDOWN = "C_SPLIT_BUTTON_DROPDOWN";
CSplitButton.prototype = {
    __proto__:CComponent.prototype,
	addActionListener:function(l){
		this.addPropertyEventListener(CSplitButton.EVENT_IDENTIFIER_ACTION,l);
	},
	removeActionListener:function(l){
		this.removePropertyEventListener(CSplitButton.EVENT_IDENTIFIER_ACTION,l);
	},
	fireActionEvent:function(){
		this.firePropertyEvent(CSplitButton.EVENT_IDENTIFIER_ACTION);
	},
	addDropdownListener:function(l){
		this.addPropertyEventListener(CSplitButton.EVENT_IDENTIFIER_DROPDOWN,l);
	},
	removeDropdownListener:function(l){
		this.removePropertyEventListener(CSplitButton.EVENT_IDENTIFIER_DROPDOWN,l);
	},
	fireDropdownEvent:function(){
		this.firePropertyEvent(CSplitButton.EVENT_IDENTIFIER_DROPDOWN);
	},
    toHTMLString:function(){
       this.addCssClassComponent(this._iconCSS);
       return "<a"+(this._insets?" style=\"margin:"+this._insets.join(" ")+"\"":"")+" id=\""+this._UIId+"\"  class=\"chromeplus-ui-split-button"+(this._cssClassComponents.length?" "+this._cssClassComponents.join(" "):"")+"\" href=\"javascript:void(0)\" title=\"\"><span class=\"chromeplus-ui-split-button-icon\"></span><span class=\"chromeplus-ui-split-button-arrow\"></span></a>";
    },
    getDOMUI:function(){
       return this._domUI;
    },
    setTooltip:function(text){
        //this._domUI.title = text;
    },
    initUI:function(){
       var _this = this;
       this._domUI = document.querySelector("#"+this._UIId);
       this._domUI.onclick = function(e){
           if(e.target.nodeName == "SPAN"){
               _this.fireDropdownEvent();
           }else{
               _this.fireActionEvent();
           }
       }
    }
}